'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var React__default = _interopDefault(React);
var PropTypes = _interopDefault(require('prop-types'));

function createCommonjsModule(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var classnames = createCommonjsModule(function (module) {
/*!
  Copyright (c) 2017 Jed Watson.
  Licensed under the MIT License (MIT), see
  http://jedwatson.github.io/classnames
*/
/* global define */

(function () {

	var hasOwn = {}.hasOwnProperty;

	function classNames () {
		var classes = [];

		for (var i = 0; i < arguments.length; i++) {
			var arg = arguments[i];
			if (!arg) continue;

			var argType = typeof arg;

			if (argType === 'string' || argType === 'number') {
				classes.push(arg);
			} else if (Array.isArray(arg) && arg.length) {
				var inner = classNames.apply(null, arg);
				if (inner) {
					classes.push(inner);
				}
			} else if (argType === 'object') {
				for (var key in arg) {
					if (hasOwn.call(arg, key) && arg[key]) {
						classes.push(key);
					}
				}
			}
		}

		return classes.join(' ');
	}

	if (module.exports) {
		classNames.default = classNames;
		module.exports = classNames;
	} else {
		window.classNames = classNames;
	}
}());
});

var SchemaContext = React__default.createContext({ values: {}, rootComponentDisplayName: null, editingComponent: null, editingComponentPath: [], htmlTypes: [], availableTypes: [], docgenInfo: null });

var UIContext = React__default.createContext({});

var text = {
  htmlAttributeDescription: 'HTML Attribute',
  customComponentMarkupHeader: 'Custom Component Markup',
  componentMarkupHeader: 'Component Markup',
  componentPreviewHeader: 'Component Preview',
  propsEditorHeader: 'Props Editor',
  propTableHeader: 'Component Prop Table',
  reset: 'Reset',
  expandCollapseAriaLabel: 'Expand/Collapse the ReactLiveProps playground',
  propertyColumnName: 'Property',
  typeColumnName: 'Type',
  descriptionColumnName: 'Description',
  required: 'required',
  noPropertiesMessage: 'There are no properties for the selected component type',
  arrayItemLabel: 'Item #',
  componentPreviewError: 'There was an error rendering the component.  Check your props and try again.'
};

var TextContext = React__default.createContext(text);

var AddButton = function AddButton(_ref) {
  var onClick = _ref.onClick,
      className = _ref.className;
  return React__default.createElement(
    UIContext.Consumer,
    null,
    function (_ref2) {
      var AddButton = _ref2.AddButton;
      return React__default.createElement(AddButton, { onClick: onClick, className: className });
    }
  );
};

AddButton.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

var DeleteButton = function DeleteButton(_ref) {
  var onClick = _ref.onClick,
      className = _ref.className;
  return React__default.createElement(
    UIContext.Consumer,
    null,
    function (_ref2) {
      var DeleteButton = _ref2.DeleteButton;
      return React__default.createElement(DeleteButton, { onClick: onClick, className: className });
    }
  );
};

DeleteButton.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

var DefaultContext = {
  color: undefined,
  size: undefined,
  className: undefined,
  style: undefined,
  attr: undefined
};
var IconContext = React__default.createContext && React__default.createContext(DefaultContext);

var __assign = undefined && undefined.__assign || function () {
  __assign = Object.assign || function (t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
      s = arguments[i];

      for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
    }

    return t;
  };

  return __assign.apply(this, arguments);
};

var __rest = undefined && undefined.__rest || function (s, e) {
  var t = {};

  for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0) t[p] = s[p];

  if (s != null && typeof Object.getOwnPropertySymbols === "function") for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0) t[p[i]] = s[p[i]];
  return t;
};

function Tree2Element(tree) {
  return tree && tree.map(function (node, i) {
    return React__default.createElement(node.tag, __assign({
      key: i
    }, node.attr), Tree2Element(node.child));
  });
}

function GenIcon(data) {
  return function (props) {
    return React__default.createElement(IconBase, __assign({
      attr: __assign({}, data.attr)
    }, props), Tree2Element(data.child));
  };
}
function IconBase(props) {
  var elem = function (conf) {
    var computedSize = props.size || conf.size || "1em";
    var className;
    if (conf.className) className = conf.className;
    if (props.className) className = (className ? className + ' ' : '') + props.className;

    var attr = props.attr,
        svgProps = __rest(props, ["attr"]);

    return React__default.createElement("svg", __assign({
      stroke: "currentColor",
      fill: "currentColor",
      strokeWidth: "0"
    }, conf.attr, attr, svgProps, {
      className: className,
      style: __assign({
        color: props.color || conf.color
      }, conf.style, props.style),
      height: computedSize,
      width: computedSize
    }), props.children);
  };

  return IconContext !== undefined ? React__default.createElement(IconContext.Consumer, null, function (conf) {
    return elem(conf);
  }) : elem(DefaultContext);
}

// THIS FILE IS AUTO GENERATED
var MdDelete = function (props) {
  return GenIcon({"tag":"svg","attr":{"viewBox":"0 0 24 24"},"child":[{"tag":"path","attr":{"d":"M6 19c0 1.1.9 2 2 2h8c1.1 0 2-.9 2-2V7H6v12zM19 4h-3.5l-1-1h-5l-1 1H5v2h14V4z"}}]})(props);
};
MdDelete.displayName = "MdDelete";
var MdAddCircle = function (props) {
  return GenIcon({"tag":"svg","attr":{"viewBox":"0 0 24 24"},"child":[{"tag":"path","attr":{"d":"M12 2C6.48 2 2 6.48 2 12s4.48 10 10 10 10-4.48 10-10S17.52 2 12 2zm5 11h-4v4h-2v-4H7v-2h4V7h2v4h4v2z"}}]})(props);
};
MdAddCircle.displayName = "MdAddCircle";
var MdSave = function (props) {
  return GenIcon({"tag":"svg","attr":{"viewBox":"0 0 24 24"},"child":[{"tag":"path","attr":{"d":"M17 3H5c-1.11 0-2 .9-2 2v14c0 1.1.89 2 2 2h14c1.1 0 2-.9 2-2V7l-4-4zm-5 16c-1.66 0-3-1.34-3-3s1.34-3 3-3 3 1.34 3 3-1.34 3-3 3zm3-10H5V5h10v4z"}}]})(props);
};
MdSave.displayName = "MdSave";
var MdExpandLess = function (props) {
  return GenIcon({"tag":"svg","attr":{"viewBox":"0 0 24 24"},"child":[{"tag":"path","attr":{"d":"M12 8l-6 6 1.41 1.41L12 10.83l4.59 4.58L18 14z"}}]})(props);
};
MdExpandLess.displayName = "MdExpandLess";
var MdExpandMore = function (props) {
  return GenIcon({"tag":"svg","attr":{"viewBox":"0 0 24 24"},"child":[{"tag":"path","attr":{"d":"M16.59 8.59L12 13.17 7.41 8.59 6 10l6 6 6-6z"}}]})(props);
};
MdExpandMore.displayName = "MdExpandMore";

var Expand = function Expand(_ref) {
  var onClick = _ref.onClick,
      className = _ref.className;
  return React__default.createElement(
    'button',
    { type: 'button', onClick: onClick, className: className, 'aria-label': 'Expand React Live Props' },
    React__default.createElement(MdExpandMore, null)
  );
};

Expand.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

var Collapse = function Collapse(_ref) {
  var onClick = _ref.onClick,
      className = _ref.className;
  return React__default.createElement(
    'button',
    { type: 'button', onClick: onClick, className: className, 'aria-label': 'Collapse React Live Props' },
    React__default.createElement(MdExpandLess, null)
  );
};

Collapse.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

var SaveButton = function SaveButton(_ref) {
  var onClick = _ref.onClick,
      className = _ref.className;
  return React__default.createElement(
    UIContext.Consumer,
    null,
    function (_ref2) {
      var SaveButton = _ref2.SaveButton;
      return React__default.createElement(SaveButton, { onClick: onClick, className: className });
    }
  );
};

SaveButton.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

var tryParseStringAsType = function tryParseStringAsType(value) {
  try {
    return JSON.parse(value);
  } catch (e) {
    return value;
  }
};

var tryConvertTypeToString = function tryConvertTypeToString(value) {
  if (typeof value === 'string') return value;

  try {
    return JSON.stringify(value);
  } catch (e) {
    return value;
  }
};

var findComponentProperties = function findComponentProperties() {
  var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  return Object.keys(props).filter(function (prop) {
    if (props[prop].type.name === 'node') return true;

    if (props[prop].type.name === 'element') return true;

    if (props[prop].type.name === 'arrayOf' && (props[prop].type.value.name === 'node' || props[prop].type.value.name === 'element')) return true;

    return false;
  });
};

var getDisplayName = function getDisplayName(value) {
  if (typeof value === 'string') return value;

  if (value.type) {
    if (typeof value.type === 'string') return value.type;

    if (value.type.toString() === 'Symbol(react.fragment)') return 'React.Fragment';

    return value.type.displayName || value.type.name || 'Component';
  }

  return value.displayName || value.name || 'Component';
};

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".PropWrapper_rlpProp__1gpby {\n  padding: 12px;\n}\n\n.PropWrapper_rlpProp__1gpby label {\n  display: block;\n}\n\n.PropWrapper_rlpPropDescription__2dyKl {\n  font-weight: normal;\n  font-size: 14px;\n  border-bottom: none;\n  margin-bottom: 0px;\n}\n\n.PropWrapper_rlpPropInput__ZLT6x input {\n  width: 100%;\n  font-weight: normal;\n}\n\n.PropWrapper_rlpPropAddButton__2QpH3,\n.PropWrapper_rlpPropDeleteButton__3bd9j {\n  margin-left: 12px;\n}\n";
var styles = { "rlpProp": "PropWrapper_rlpProp__1gpby", "rlpPropDescription": "PropWrapper_rlpPropDescription__2dyKl", "rlpPropInput": "PropWrapper_rlpPropInput__ZLT6x", "rlpPropAddButton": "PropWrapper_rlpPropAddButton__2QpH3", "rlpPropDeleteButton": "PropWrapper_rlpPropDeleteButton__3bd9j" };
styleInject(css);

var buildName = function buildName(name, text) {
  if (typeof name === 'number') {
    return '' + text.arrayItemLabel + name;
  }

  return name;
};

var PropWrapper = function PropWrapper(_ref) {
  var children = _ref.children,
      description = _ref.description,
      name = _ref.name,
      onAdd = _ref.onAdd,
      onDelete = _ref.onDelete;
  return React__default.createElement(
    TextContext.Consumer,
    null,
    function (text) {
      return React__default.createElement(
        'div',
        { className: classnames('rlpProp', styles.rlpProp) },
        name !== '' && React__default.createElement(
          'div',
          { className: classnames('rlpPropHeader', styles.rlpPropHeader) },
          React__default.createElement(
            'strong',
            { className: classnames('rlpPropName', styles.rlpPropName) },
            buildName(name, text),
            typeof onAdd === 'function' && React__default.createElement(AddButton, { onClick: onAdd, className: classnames('rlpPropAddButton', styles.rlpPropAddButton) }),
            typeof onDelete === 'function' && React__default.createElement(DeleteButton, { onClick: onDelete, className: classnames('rlpPropDeleteButton', styles.rlpPropDeleteButton) })
          ),
          description && React__default.createElement(
            'legend',
            { className: classnames('rlpPropDescription', styles.rlpPropDescription) },
            description
          )
        ),
        React__default.createElement(
          'div',
          { className: classnames('rlpPropInput', styles.rlpPropInput) },
          children
        )
      );
    }
  );
};

PropWrapper.propTypes = {
  children: PropTypes.oneOfType([PropTypes.string, PropTypes.func, PropTypes.node]),
  description: PropTypes.string,
  name: PropTypes.oneOfType([PropTypes.string, PropTypes.number]),
  onAdd: PropTypes.func,
  onDelete: PropTypes.func
};

var css$1 = ".AddHtmlAttribute_attributeForm__EubSa {\n  display: flex;\n  min-height: 0px;\n  flex-direction: row;\n}\n\n.AddHtmlAttribute_attributeForm__EubSa > * {\n  flex-grow: 1;\n  flex-shrink: 1;\n}\n\n.AddHtmlAttribute_attributeForm__EubSa > button {\n  flex-grow: 0;\n  flex-basis: 0;\n  outline: none;\n  appearance: none;\n  border: none;\n  background: none;\n}\n\n.AddHtmlAttribute_attributeForm__EubSa button:hover {\n  background-color: lightgray;\n}\n";
var styles$1 = { "attributeForm": "AddHtmlAttribute_attributeForm__EubSa" };
styleInject(css$1);

var AddHtmlAttribute = function AddHtmlAttribute(_ref) {
  var pendingAttributeName = _ref.pendingAttributeName,
      pendingAttributeValue = _ref.pendingAttributeValue,
      onAddProperty = _ref.onAddProperty,
      _onChange = _ref.onChange;
  return React__default.createElement(
    SchemaContext.Consumer,
    null,
    function (_ref2) {
      var editingComponent = _ref2.editingComponent,
          values = _ref2.values,
          editingComponentPath = _ref2.editingComponentPath;
      return React__default.createElement(
        PropWrapper,
        { name: '' },
        React__default.createElement(
          'div',
          { className: styles$1.attributeForm },
          React__default.createElement('input', { type: 'text', value: pendingAttributeName, placeholder: 'Attribute name', onChange: function onChange(e) {
              return _onChange({ pendingAttributeName: e.target.value });
            } }),
          React__default.createElement('input', { type: 'text', value: pendingAttributeValue, placeholder: 'Attribute value', onChange: function onChange(e) {
              return _onChange({ pendingAttributeValue: e.target.value });
            } }),
          React__default.createElement(AddButton, { onClick: function onClick() {
              return onAddProperty(editingComponent, editingComponentPath, values, pendingAttributeName, tryParseStringAsType(pendingAttributeValue));
            } })
        )
      );
    }
  );
};

AddHtmlAttribute.propTypes = {
  pendingAttributeName: PropTypes.string.isRequired,
  pendingAttributeValue: PropTypes.string.isRequired,
  onAddProperty: PropTypes.func.isRequired,
  onChange: PropTypes.func.isRequired
};

var css$2 = ".styles_rlpPropsTable__33M9a table {\n  width: 100%;\n  box-shadow: 0 0 0 1px #CED4DE;\n  border-radius: 4px;\n  border-collapse: collapse;\n  border-style: hidden;\n  border-spacing: 0px;\n}\n\n.styles_rlpPropsTable__33M9a thead {\n  background: #EEF1F5\n}\n\n.styles_rlpPropsTable__33M9a tbody > tr {\n  border-top: 1px solid #CED4DE;\n}\n\n.styles_rlpPropsTable__33M9a th,\n.styles_rlpPropsTable__33M9a td {\n  padding: 16px;\n}\n\n.styles_rlpRequired__98w-B {\n  margin-left: 8px;\n  font-weight: bold;\n}\n\n.styles_rlpPropName__PKuCV {}\n\n.styles_rlpTypeName__ebU-D {}\n\n.styles_rlpDescription__22S63 {\n  width: 50%;\n}\n\n.styles_rlpNoAvailableProperties__5AfHZ {\n  padding: 16px;\n}\n\n.styles_rlpComponentSelector__lbc0y {\n  margin: 8px 0px;\n}\n\n.styles_rlpComponentSelector__lbc0y select {\n  margin-left: 8px;\n}\n\n.styles_rlpTypeNameAction__XPeoU {\n  float: right;\n  background-color: transparent;\n  border: none;\n}\n\n.styles_rlpPropertyTypeDetails__1Eu1F {\n  margin-top: 8px;\n}\n";
var styles$2 = { "rlpPropsTable": "styles_rlpPropsTable__33M9a", "rlpRequired": "styles_rlpRequired__98w-B", "rlpPropName": "styles_rlpPropName__PKuCV", "rlpTypeName": "styles_rlpTypeName__ebU-D", "rlpDescription": "styles_rlpDescription__22S63", "rlpNoAvailableProperties": "styles_rlpNoAvailableProperties__5AfHZ", "rlpComponentSelector": "styles_rlpComponentSelector__lbc0y", "rlpTypeNameAction": "styles_rlpTypeNameAction__XPeoU", "rlpPropertyTypeDetails": "styles_rlpPropertyTypeDetails__1Eu1F" };
styleInject(css$2);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var TYPES_WITH_DETAILS = ['enum', 'arrayOf', 'shape'];

var PropsTable = function (_React$Component) {
  inherits(PropsTable, _React$Component);

  function PropsTable() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, PropsTable);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = PropsTable.__proto__ || Object.getPrototypeOf(PropsTable)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      selectedComponentDisplayName: null,
      expandedProperties: {}
    }, _this.updateComponent = function (e) {
      _this.setState({
        selectedComponentDisplayName: e.target.value,
        expandedProperties: {}
      });
    }, _this.expandProperty = function (propName) {
      _this.setState({
        expandedProperties: _extends({}, _this.state.expandedProperties, defineProperty({}, propName, true))
      });
    }, _this.collapseProperty = function (propName) {
      var _this$state$expandedP = _this.state.expandedProperties,
          propToRemove = _this$state$expandedP[propName],
          rest = objectWithoutProperties(_this$state$expandedP, [propName]);

      _this.setState({
        expandedProperties: _extends({}, rest)
      });
    }, _this.getEnumDetails = function (values) {
      return values.map(function (value) {
        return value.value;
      }).join(', ');
    }, _this.getArrayOfDetails = function (values) {
      if (values.name === 'shape') {
        return 'ArrayOf<' + _this.getShapeDetails(values.value) + '>';
      }

      if (values.name === 'arrayOf') {
        return 'ArrayOf<' + _this.getArrayOfDetails(values.value) + '>';
      }

      return 'ArrayOf<\'' + values.name + '\'>';
    }, _this.getShapeDetails = function (values) {
      return 'Shape<{ ' + Object.keys(values).map(function (key) {
        return '\'' + key + '\': \'' + values[key].name + '\'';
      }).join(', ') + ' }>';
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(PropsTable, [{
    key: 'render',
    value: function render() {
      var _this2 = this;

      return React__default.createElement(
        'div',
        { className: classnames('rlpPropsTable', styles$2.rlpPropsTable) },
        React__default.createElement(
          TextContext.Consumer,
          null,
          function (text) {
            return React__default.createElement(
              SchemaContext.Consumer,
              null,
              function (_ref2) {
                var rootComponentDisplayName = _ref2.rootComponentDisplayName,
                    docgenInfo = _ref2.docgenInfo;

                var currentComponent = _this2.state.selectedComponentDisplayName || rootComponentDisplayName;
                var componentProps = docgenInfo[currentComponent].props;
                return React__default.createElement(
                  React__default.Fragment,
                  null,
                  componentProps && Object.keys(componentProps).length > 0 && React__default.createElement(
                    'table',
                    null,
                    React__default.createElement(
                      'thead',
                      null,
                      React__default.createElement(
                        'tr',
                        null,
                        React__default.createElement(
                          'th',
                          { className: classnames('rlpPropName', styles$2.rlpPropName) },
                          text.propertyColumnName
                        ),
                        React__default.createElement(
                          'th',
                          { className: classnames('rlpTypeName', styles$2.rlpTypeName) },
                          text.typeColumnName
                        ),
                        React__default.createElement(
                          'th',
                          { className: classnames('rlpDescription', styles$2.rlpDescription) },
                          text.descriptionColumnName
                        )
                      )
                    ),
                    React__default.createElement(
                      'tbody',
                      null,
                      Object.keys(componentProps).map(function (propName) {
                        var property = componentProps[propName];
                        return React__default.createElement(
                          'tr',
                          { key: propName },
                          React__default.createElement(
                            'td',
                            { className: classnames('rlpPropName', styles$2.rlpPropName) },
                            propName,
                            property.required && React__default.createElement(
                              'span',
                              { className: classnames('rlpRequired', styles$2.rlpRequired) },
                              '(',
                              text.required,
                              ')'
                            )
                          ),
                          React__default.createElement(
                            'td',
                            { className: classnames('rlpTypeName', styles$2.rlpTypeName) },
                            React__default.createElement(
                              'div',
                              null,
                              property.type.name,
                              TYPES_WITH_DETAILS.includes(property.type.name) && !_this2.state.expandedProperties[propName] && React__default.createElement(Expand, { className: classnames('rlpTypeNameAction', styles$2.rlpTypeNameAction), onClick: function onClick() {
                                  return _this2.expandProperty(propName);
                                } }),
                              TYPES_WITH_DETAILS.includes(property.type.name) && _this2.state.expandedProperties[propName] && React__default.createElement(Collapse, { className: classnames('rlpTypeNameAction', styles$2.rlpTypeNameAction), onClick: function onClick() {
                                  return _this2.collapseProperty(propName);
                                } })
                            ),
                            _this2.state.expandedProperties[propName] && React__default.createElement(
                              'div',
                              { className: classnames('rlpPropertyTypeDetails', styles$2.rlpPropertyTypeDetails) },
                              property.type.name === 'enum' && React__default.createElement(
                                'em',
                                null,
                                _this2.getEnumDetails(property.type.value)
                              ),
                              property.type.name === 'arrayOf' && React__default.createElement(
                                'em',
                                null,
                                _this2.getArrayOfDetails(property.type.value)
                              ),
                              property.type.name === 'shape' && React__default.createElement(
                                'em',
                                null,
                                _this2.getShapeDetails(property.type.value)
                              )
                            )
                          ),
                          React__default.createElement(
                            'td',
                            { className: classnames('rlpDescription', styles$2.rlpDescription) },
                            property.description
                          )
                        );
                      })
                    )
                  ),
                  (!componentProps || Object.keys(componentProps).length === 0) && React__default.createElement(
                    'div',
                    { className: classnames('rlpNoAvailableProperties', styles$2.rlpNoAvailableProperties) },
                    React__default.createElement(
                      'em',
                      null,
                      text.noPropertiesMessage
                    )
                  )
                );
              }
            );
          }
        )
      );
    }
  }]);
  return PropsTable;
}(React__default.Component);

var css$3 = "textarea.base_rlpPropField__Q9H0f,\nselect.base_rlpPropField__Q9H0f {\n  width: 100%;\n  font-weight: normal;\n}\n\n.base_rlpPropField__Q9H0f select {\n  width: 120px;\n}\n\n.base_rlpPropField__Q9H0f input {\n  width: calc(100% - 120px);\n}\n";
var styles$3 = { "rlpPropField": "base_rlpPropField__Q9H0f" };
styleInject(css$3);

var StringFieldRenderer = function StringFieldRenderer(_ref) {
  var name = _ref.name,
      displayName = _ref.displayName,
      value = _ref.value,
      property = _ref.property,
      _onChange = _ref.onChange,
      onDelete = _ref.onDelete;

  var valueOrDefault = value || '';
  return React__default.createElement(
    PropWrapper,
    { name: displayName, description: property.description, onDelete: onDelete },
    React__default.createElement('input', {
      type: 'text',
      value: valueOrDefault,
      className: classnames('rlpPropField', styles$3.rlpPropField),
      onChange: function onChange(e) {
        _onChange(name, e.target.value);
      }
    })
  );
};

StringFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  value: PropTypes.any,
  onChange: PropTypes.func.isRequired,
  onDelete: PropTypes.func.isRequired,
  property: PropTypes.object.isRequired,
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};

var NumberFieldRenderer = function NumberFieldRenderer(_ref) {
  var name = _ref.name,
      displayName = _ref.displayName,
      value = _ref.value,
      property = _ref.property,
      _onChange = _ref.onChange,
      onDelete = _ref.onDelete;

  var valueOrDefault = value || '';
  return React__default.createElement(
    PropWrapper,
    { name: displayName, description: property.description, onDelete: onDelete },
    React__default.createElement('input', {
      type: 'number',
      value: valueOrDefault,
      className: classnames('rlpPropField', styles$3.rlpPropField),
      onChange: function onChange(e) {
        if (e.target.value.indexOf('.') > -1) {
          _onChange(name, parseFloat(e.target.value, 10));
        } else {
          _onChange(name, parseInt(e.target.value, 10));
        }
      }
    })
  );
};

NumberFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  value: PropTypes.any,
  onChange: PropTypes.func.isRequired,
  onDelete: PropTypes.func.isRequired,
  property: PropTypes.object.isRequired,
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};

var BoolFieldRenderer = function BoolFieldRenderer(_ref) {
  var name = _ref.name,
      displayName = _ref.displayName,
      property = _ref.property,
      value = _ref.value,
      _onChange = _ref.onChange,
      onDelete = _ref.onDelete;

  var valueOrDefault = value || false;

  return React__default.createElement(
    PropWrapper,
    { name: displayName, description: property.description, onDelete: onDelete },
    React__default.createElement('input', {
      type: 'checkbox',
      checked: valueOrDefault,
      className: classnames('rlpPropField', styles$3.rlpPropField),
      onChange: function onChange(e) {
        _onChange(name, e.target.checked);
      }
    })
  );
};

BoolFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  value: PropTypes.any,
  onChange: PropTypes.func.isRequired,
  onDelete: PropTypes.func.isRequired,
  property: PropTypes.object.isRequired,
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};

var EnumFieldRenderer = function EnumFieldRenderer(_ref) {
  var name = _ref.name,
      displayName = _ref.displayName,
      property = _ref.property,
      value = _ref.value,
      _onChange = _ref.onChange,
      onDelete = _ref.onDelete;

  var valueOrDefault = value || '';
  return React__default.createElement(
    PropWrapper,
    { name: displayName, description: property.description, onDelete: true },
    React__default.createElement(
      'select',
      {
        value: valueOrDefault,
        className: classnames('rlpPropField', styles$3.rlpPropField),
        onChange: function onChange(e) {
          var newValue = e.target.value === '' ? undefined : e.target.value;
          _onChange(name, newValue);
        }
      },
      React__default.createElement('option', { value: '' }),
      property.type.value.map(function (option) {
        var optionValue = option.value;
        if (typeof optionValue === 'string') {
          if (optionValue.indexOf('\'') === 0 || optionValue.indexOf('"') === 0) {
            optionValue = optionValue.split('').filter(function (_, idx) {
              return idx !== 0 && idx !== optionValue.length - 1;
            }).join('');
          }
        }
        return React__default.createElement(
          'option',
          { key: optionValue, value: optionValue },
          optionValue
        );
      })
    )
  );
};

EnumFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  property: PropTypes.object.isRequired,
  value: PropTypes.any,
  onChange: PropTypes.func.isRequired,
  onDelete: PropTypes.func.isRequired,
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};

var RendererContext = React__default.createContext({});

var RendererResolver = function RendererResolver(_ref) {
  var type = _ref.type,
      value = _ref.value,
      rest = objectWithoutProperties(_ref, ['type', 'value']);

  return React__default.createElement(
    RendererContext.Consumer,
    null,
    function (renderers) {
      if (type.name === 'union') {
        var Renderer = renderers[type.name];
        var valueOrDefault = Renderer.getValueWithDefault ? Renderer.getValueWithDefault(value) : value;

        return React__default.createElement(Renderer, _extends({ renderers: renderers }, rest, { type: type.value, value: valueOrDefault }));
      }

      if (renderers[type.name]) {
        var _Renderer = renderers[type.name];
        var _valueOrDefault = _Renderer.getValueWithDefault ? _Renderer.getValueWithDefault(value) : value;

        return React__default.createElement(_Renderer, _extends({}, rest, { type: type.value, value: _valueOrDefault }));
      }

      console.log('UNSUPPORTED', type);
      return React__default.createElement(
        PropWrapper,
        { name: rest.name, description: rest.property.description },
        'PROP TYPE NOT SUPPORTED: ',
        type.value && type.value.name ? type.name + ' (' + type.value.name + ')' : type.name
      );
    }
  );
};

RendererResolver.propTypes = {
  type: PropTypes.shape({
    name: PropTypes.string
  }),
  value: PropTypes.any
};

var css$4 = ".Array_arrayItem__3gqZF {\n  display: flex;\n  min-height: 0px;\n  flex-direction: row;\n  flex-wrap: wrap;\n}\n\n.Array_arrayItem__3gqZF > * {\n  flex-grow: 1;\n  flex-shrink: 1;\n  flex-basis: 250px;\n}\n\n.Array_arrayItem__3gqZF button {\n  flex-grow: 0;\n  flex-basis: 0;\n}\n\n.Array_arrayItem__3gqZF button:hover {\n  background-color: lightgray;\n}\n";
var arrayStyles = { "arrayItem": "Array_arrayItem__3gqZF" };
styleInject(css$4);

var emptyProperty = {
  description: null
};

var ArrayOfFieldRenderer = function ArrayOfFieldRenderer(_ref) {
  var name = _ref.name,
      displayName = _ref.displayName,
      type = _ref.type,
      property = _ref.property,
      value = _ref.value,
      _onChange = _ref.onChange,
      onDelete = _ref.onDelete;

  var valueOrDefault = typeof value === 'string' ? [value] : value || [];

  return React__default.createElement(
    PropWrapper,
    {
      name: displayName,
      description: property.description,
      onAdd: function onAdd() {
        _onChange(name, [].concat(toConsumableArray(valueOrDefault), [null]));
      },
      onDelete: onDelete
    },
    valueOrDefault.map(function (item, idx) {
      return React__default.createElement(
        'div',
        { className: classnames('rlpArrayItem', arrayStyles.arrayItem), key: name + '-' + idx },
        React__default.createElement(RendererResolver, {
          name: name,
          displayName: idx,
          type: type,
          property: emptyProperty,
          onChange: function onChange(name, updatedValue) {
            var updatedValues = value.map(function (origValue, itemIndex) {
              if (itemIndex === idx) {
                return updatedValue;
              }

              return origValue;
            });
            _onChange(name, updatedValues);
          },
          onDelete: function onDelete() {
            var updatedValues = value.filter(function (_, itemIndex) {
              if (itemIndex === idx) return false;

              return true;
            });
            _onChange(name, updatedValues);
          },
          value: item
        })
      );
    })
  );
};

ArrayOfFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  property: PropTypes.object,
  value: PropTypes.oneOfType([PropTypes.array, PropTypes.string]),
  onChange: PropTypes.func.isRequired,
  onDelete: PropTypes.func.isRequired,
  type: PropTypes.shape({
    name: PropTypes.string
  }),
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};

ArrayOfFieldRenderer.getValueWithDefault = function (value) {
  if (value) return value;

  return [];
};

var ArrayFieldRenderer = function ArrayFieldRenderer(_ref) {
  var name = _ref.name,
      displayName = _ref.displayName,
      _ref$type = _ref.type,
      type = _ref$type === undefined ? { name: 'any' } : _ref$type,
      property = _ref.property,
      value = _ref.value,
      onChange = _ref.onChange;

  return React__default.createElement(ArrayOfFieldRenderer, { name: name, displayName: displayName, type: type, property: property, value: value, onChange: onChange });
};

ArrayFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  property: PropTypes.object,
  value: PropTypes.oneOfType([PropTypes.array, PropTypes.string]),
  onChange: PropTypes.func.isRequired,
  type: PropTypes.shape({
    name: PropTypes.string
  }),
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};

ArrayFieldRenderer.getValueWithDefault = function (value) {
  if (value) return value;

  return [];
};

var ShapeFieldRenderer = function ShapeFieldRenderer(_ref) {
  var name = _ref.name,
      displayName = _ref.displayName,
      type = _ref.type,
      property = _ref.property,
      value = _ref.value,
      _onChange = _ref.onChange,
      onDelete = _ref.onDelete;

  var keys = Object.keys(type);
  var valueOrDefault = value || {};

  return React__default.createElement(
    PropWrapper,
    { name: displayName, description: property.description, onDelete: onDelete },
    keys.map(function (item) {
      return React__default.createElement(RendererResolver, {
        key: name + '-' + item,
        name: item,
        displayName: item,
        type: type[item],
        property: type[item],
        onChange: function onChange(_, newPropValue) {
          var newValue = _extends({}, valueOrDefault, defineProperty({}, item, newPropValue));
          _onChange(name, newValue);
        },
        value: valueOrDefault[item]
      });
    })
  );
};

ShapeFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  type: PropTypes.object,
  property: PropTypes.object,
  value: PropTypes.any,
  onChange: PropTypes.func.isRequired,
  onDelete: PropTypes.func.isRequired,
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};

ShapeFieldRenderer.getValueWithDefault = function (value) {
  if (value) return value;

  return {};
};

var AnyFieldRenderer = function AnyFieldRenderer(_ref) {
  var name = _ref.name,
      displayName = _ref.displayName,
      property = _ref.property,
      value = _ref.value,
      _onChange = _ref.onChange,
      onDelete = _ref.onDelete;

  var valueOrDefault = value || '';
  return React__default.createElement(
    PropWrapper,
    { name: displayName, description: property.description, onDelete: onDelete },
    React__default.createElement('input', {
      type: 'text',
      value: tryConvertTypeToString(valueOrDefault),
      className: classnames('rlpPropField', styles$3.rlpPropField),
      onChange: function onChange(e) {
        _onChange(name, tryParseStringAsType(e.target.value));
      }
    })
  );
};

AnyFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  property: PropTypes.object.isRequired,
  value: PropTypes.any,
  onChange: PropTypes.func.isRequired,
  onDelete: PropTypes.func.isRequired,
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};

var NodeFieldRenderer = function (_React$Component) {
  inherits(NodeFieldRenderer, _React$Component);

  function NodeFieldRenderer(props, context) {
    classCallCheck(this, NodeFieldRenderer);

    var _this = possibleConstructorReturn(this, (NodeFieldRenderer.__proto__ || Object.getPrototypeOf(NodeFieldRenderer)).call(this, props, context));

    var htmlDictionary = {};
    props.htmlTypes.forEach(function (type) {
      var typeName = getDisplayName(type);
      htmlDictionary[typeName] = type;
    });

    var typeDictionary = {};
    props.availableTypes.filter(function (type) {
      return typeof htmlDictionary[type] === 'undefined';
    }).forEach(function (type) {
      var typeName = getDisplayName(type);
      if (typeName !== 'React.Fragment') {
        typeDictionary[typeName] = type;
      }
    });

    var currentTypeName = getDisplayName(props.value || '');
    var currentElementType = null;
    if (props.htmlTypes.includes(currentTypeName)) {
      currentElementType = 'HTML';
    } else if (currentTypeName === 'React.Fragment') {
      currentElementType = 'React';
    } else if (typeDictionary[currentTypeName]) {
      currentElementType = 'Component';
    } else {
      currentElementType = 'String';
    }

    _this.state = {
      htmlDictionary: htmlDictionary,
      typeDictionary: typeDictionary,
      currentElementType: currentElementType
    };
    return _this;
  }

  createClass(NodeFieldRenderer, [{
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          name = _props.name,
          displayName = _props.displayName,
          value = _props.value,
          _onChange = _props.onChange,
          onDelete = _props.onDelete,
          property = _props.property;


      var currentTypeName = getDisplayName(value || '');
      return React__default.createElement(
        PropWrapper,
        { name: displayName, description: property.description, onDelete: onDelete },
        React__default.createElement(
          'select',
          {
            className: classnames('rlpPropField', styles$3.rlpPropField),
            value: this.state.currentElementType,
            onChange: function onChange(e) {
              _this2.setState({
                currentElementType: e.target.value
              });
            }
          },
          React__default.createElement(
            'option',
            { value: 'HTML' },
            'HTML Element'
          ),
          React__default.createElement(
            'option',
            { value: 'Component' },
            'Application Component'
          ),
          React__default.createElement(
            'option',
            { value: 'React' },
            'Native React Element'
          ),
          React__default.createElement(
            'option',
            { valye: 'String' },
            'String'
          )
        ),
        this.state.currentElementType === 'HTML' && React__default.createElement(
          'select',
          {
            value: currentTypeName,
            className: classnames('rlpPropField', styles$3.rlpPropField),
            onChange: function onChange(e) {
              var newTypeName = e.target.value === '' ? null : e.target.value;
              if (newTypeName === null) {
                _onChange(name, null);
                return;
              }

              _onChange(name, React__default.createElement(_this2.state.htmlDictionary[newTypeName]));
            }
          },
          React__default.createElement(
            'option',
            { value: '' },
            'UNSET'
          ),
          Object.keys(this.state.htmlDictionary).map(function (typeName) {
            return React__default.createElement(
              'option',
              { key: typeName, value: typeName },
              typeName
            );
          })
        ),
        this.state.currentElementType === 'Component' && React__default.createElement(
          'select',
          {
            value: currentTypeName,
            className: classnames('rlpPropField', styles$3.rlpPropField),
            onChange: function onChange(e) {
              var newTypeName = e.target.value === '' ? null : e.target.value;
              if (newTypeName === null) {
                _onChange(name, null);
                return;
              }

              _onChange(name, React__default.createElement(_this2.state.typeDictionary[newTypeName]));
            }
          },
          React__default.createElement(
            'option',
            { value: '' },
            'UNSET'
          ),
          Object.keys(this.state.typeDictionary).map(function (typeName) {
            return React__default.createElement(
              'option',
              { key: typeName, value: typeName },
              typeName
            );
          })
        ),
        this.state.currentElementType === 'React' && React__default.createElement(
          'select',
          {
            value: currentTypeName,
            className: classnames('rlpPropField', styles$3.rlpPropField),
            onChange: function onChange(e) {
              var newTypeName = e.target.value === '' ? null : e.target.value;
              if (newTypeName === 'React.Fragment') {
                _onChange(name, React__default.createElement(React__default.Fragment));
                return;
              }

              if (newTypeName === null) {
                _onChange(name, null);
              }
            }
          },
          React__default.createElement(
            'option',
            { value: '' },
            'UNSET'
          ),
          React__default.createElement(
            'option',
            { value: 'React.Fragment' },
            'React.Fragment'
          )
        ),
        this.state.currentElementType === 'String' && React__default.createElement('textarea', {
          type: 'text',
          onChange: function onChange(e) {
            _onChange(name, e.target.value);
          },
          placeholder: 'String contents',
          value: typeof value !== 'string' ? '' : value,
          className: classnames('rlpPropField', styles$3.rlpPropField)
        })
      );
    }
  }]);
  return NodeFieldRenderer;
}(React__default.Component);

NodeFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  value: PropTypes.any,
  onChange: PropTypes.func.isRequired,
  onDelete: PropTypes.func.isRequired,
  htmlTypes: PropTypes.arrayOf(PropTypes.string),
  property: PropTypes.object.isRequired,
  availableTypes: PropTypes.array,
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired])
};


var NodeFieldRendererWithContext = function NodeFieldRendererWithContext(props) {
  return React__default.createElement(
    SchemaContext.Consumer,
    null,
    function (_ref) {
      var availableTypes = _ref.availableTypes,
          htmlTypes = _ref.htmlTypes;
      return React__default.createElement(NodeFieldRenderer, _extends({}, props, { availableTypes: availableTypes, htmlTypes: htmlTypes }));
    }
  );
};

var typeofToReactType = function typeofToReactType(jsType) {
  if (jsType === 'boolean') return 'bool';

  if (jsType === 'undefined') return 'string';

  return jsType;
};

var UnionFieldRenderer = function (_React$Component) {
  inherits(UnionFieldRenderer, _React$Component);

  function UnionFieldRenderer() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, UnionFieldRenderer);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = UnionFieldRenderer.__proto__ || Object.getPrototypeOf(UnionFieldRenderer)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      currentType: typeofToReactType(typeof value === 'undefined' ? 'undefined' : _typeof(value)) || 'string'
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(UnionFieldRenderer, [{
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          renderers = _props.renderers,
          type = _props.type,
          value = _props.value,
          name = _props.name,
          displayName = _props.displayName,
          property = _props.property,
          onDelete = _props.onDelete,
          rest = objectWithoutProperties(_props, ['renderers', 'type', 'value', 'name', 'displayName', 'property', 'onDelete']);


      if (type.filter(function (allowedType) {
        return allowedType.name === 'node' || allowedType.name === 'element';
      }).length > 0) {
        var Renderer = renderers.node;
        var valueOrDefault = Renderer.getValueWithDefault ? Renderer.getValueWithDefault(value) : value;

        return React__default.createElement(Renderer, _extends({}, rest, { name: name, displayName: displayName, property: property, type: type, value: valueOrDefault }));
      }

      return React__default.createElement(
        PropWrapper,
        { name: displayName, description: property.description, onDelete: onDelete },
        React__default.createElement(
          'div',
          null,
          React__default.createElement(
            'select',
            { value: this.state.currentType, onChange: function onChange(e) {
                _this2.setState({
                  currentType: e.target.value
                });
              } },
            type.map(function (allowedType) {
              return React__default.createElement(
                'option',
                { value: allowedType.name, key: allowedType.name },
                allowedType.name
              );
            })
          ),
          type.filter(function (allowedType) {
            return allowedType.name === _this2.state.currentType;
          }).map(function (allowedType) {
            var Renderer = renderers[allowedType.name];
            var valueOrDefault = Renderer.getValueWithDefault ? Renderer.getValueWithDefault(value) : value;
            return React__default.createElement(Renderer, _extends({}, rest, { name: name, displayName: '', property: property, type: type, value: valueOrDefault, key: Renderer.displayName }));
          })
        )
      );
    }
  }]);
  return UnionFieldRenderer;
}(React__default.Component);

UnionFieldRenderer.propTypes = {
  name: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  renderers: PropTypes.object,
  value: PropTypes.oneOfType([PropTypes.array, PropTypes.string]),
  type: PropTypes.arrayOf(PropTypes.shape({
    name: PropTypes.string
  })),
  property: PropTypes.object.isRequired,
  displayName: PropTypes.oneOfType([PropTypes.string.isRequired, PropTypes.number.isRequired]),
  onDelete: PropTypes.func
};

var Renderers = {
  arrayOf: ArrayOfFieldRenderer,
  array: ArrayFieldRenderer,
  bool: BoolFieldRenderer,
  enum: EnumFieldRenderer,
  number: NumberFieldRenderer,
  shape: ShapeFieldRenderer,
  string: StringFieldRenderer,
  any: AnyFieldRenderer,
  node: NodeFieldRendererWithContext,
  element: NodeFieldRendererWithContext,
  union: UnionFieldRenderer
};

var css$5 = ".styles_rlpEditableProps__2JJuc {\n\n}\n\n.styles_rlpEditablePropsDivider__34ou1 {\n  height: 2px;\n  background-color: #e5e5e5;\n  width: 100%;\n}\n";
var styles$4 = { "rlpEditableProps": "styles_rlpEditableProps__2JJuc", "rlpEditablePropsDivider": "styles_rlpEditablePropsDivider__34ou1" };
styleInject(css$5);

var updateComponentTree = function updateComponentTree(rootComponent, editingComponentPath, propName, newValue) {
  var editedComponent = editingComponentPath.reduce(function (prev, current) {
    return prev[current];
  }, rootComponent);
  var updatedComponent = React__default.cloneElement(editedComponent, defineProperty({}, propName, newValue));

  var componentList = [];
  var workingComponent = rootComponent;
  var currentPropValue = rootComponent;
  var workingPath = [];
  editingComponentPath.forEach(function (pathItem) {
    currentPropValue = currentPropValue[pathItem];
    workingPath.push(pathItem);

    if (currentPropValue['$$typeof']) {
      componentList.push({
        component: workingComponent,
        path: workingPath
      });

      workingComponent = currentPropValue;
      workingPath = [];
    }
  });

  var currentUpdatedComponent = updatedComponent;
  for (var i = 1; i <= componentList.length; i++) {
    var item = componentList[componentList.length - i];
    // strip off the props path item at the beginning
    var propList = item.path.slice(1);
    if (propList.length === 1) {
      // a propList of length one means it is an object prop with no index
      currentUpdatedComponent = React__default.cloneElement(item.component, defineProperty({}, propList[0], currentUpdatedComponent));
    } else if (propList.length === 2) {
      // a propList of length one means it is an object prop with no index
      var newArrayValues = [].concat(toConsumableArray(item.component.props[propList[0]]));
      newArrayValues[propList[1]] = currentUpdatedComponent;
      currentUpdatedComponent = React__default.cloneElement(item.component, defineProperty({}, propList[0], newArrayValues));
    } else {
      console.error('Invalid prop list length when updating components', item);
      throw new Error('Invalid prop list when updating components');
    }
  }

  return currentUpdatedComponent;
};

var EditablePropsTable = function (_Component) {
  inherits(EditablePropsTable, _Component);

  function EditablePropsTable() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, EditablePropsTable);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = EditablePropsTable.__proto__ || Object.getPrototypeOf(EditablePropsTable)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      pendingAttributeName: '',
      pendingAttributeValue: ''
    }, _this.state = {
      pendingAttributeName: '',
      pendingAttributeValue: ''
    }, _this._onAddProperty = function (editingComponent, editingComponentPath, values, pendingAttributeName, value) {
      var updatedRootComponent = updateComponentTree(values, editingComponentPath, pendingAttributeName, value);

      _this.props.onAddProperty(editingComponent, pendingAttributeName);
      _this.props.onChange(updatedRootComponent);

      _this.setState(function (state) {
        return _extends({}, state, {
          pendingAttributeName: '',
          pendingAttributeValue: ''
        });
      });
    }, _this._onChangeProperty = function (state) {
      _this.setState(state);
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(EditablePropsTable, [{
    key: 'filterProperties',
    value: function filterProperties(docgenInfo) {
      var blacklistedProperties = this.props.blacklistedProperties || [];

      var editableProperties = this.props.editableProperties || Object.keys(docgenInfo.props);

      var editablePropertyDefs = {};
      editableProperties.forEach(function (key) {
        if (blacklistedProperties.includes(key)) return;

        editablePropertyDefs[key] = docgenInfo.props[key];
      });

      if (docgenInfo && docgenInfo.props) {
        var keys = Object.keys(docgenInfo.props);
        var editableKeys = Object.keys(editablePropertyDefs);

        if (keys.length === editableKeys.length) return docgenInfo.props;
      }

      return editablePropertyDefs;
    }
  }, {
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps, nextState) {
      var shouldUpdate = false;

      Object.entries(this.props).forEach(function (_ref2) {
        var _ref3 = slicedToArray(_ref2, 2),
            key = _ref3[0],
            val = _ref3[1];

        if (nextProps[key] !== val) {
          shouldUpdate = true;
        }
      });
      Object.entries(this.state).forEach(function (_ref4) {
        var _ref5 = slicedToArray(_ref4, 2),
            key = _ref5[0],
            val = _ref5[1];

        if (nextState[key] !== val) {
          shouldUpdate = true;
        }
      });

      if (Object.entries(this.props).length !== Object.entries(nextProps).length) {
        shouldUpdate = true;
      }

      if (Object.entries(this.state).length !== Object.entries(nextState).length) {
        shouldUpdate = true;
      }

      return shouldUpdate;
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          _ = _props.onChange,
          className = _props.className,
          editableProperties = _props.editableProperties,
          blacklistedProperties = _props.blacklistedProperties,
          onAddProperty = _props.onAddProperty,
          rest = objectWithoutProperties(_props, ['onChange', 'className', 'editableProperties', 'blacklistedProperties', 'onAddProperty']);


      return React__default.createElement(
        SchemaContext.Consumer,
        null,
        function (_ref6) {
          var values = _ref6.values,
              editingComponent = _ref6.editingComponent,
              editingComponentPath = _ref6.editingComponentPath,
              htmlTypes = _ref6.htmlTypes,
              docgenInfo = _ref6.docgenInfo;

          var filteredSchema = _this2.filterProperties(docgenInfo[editingComponent]);
          var propertyKeys = Object.keys(filteredSchema);
          var currentComponent = editingComponentPath.reduce(function (prev, current) {
            return prev[current];
          }, values);

          var onChange = function onChange(propName, newValue) {
            var updatedRootComponent = updateComponentTree(values, editingComponentPath, propName, newValue);

            _this2.props.onChange(updatedRootComponent);
          };

          var componentPropKeys = Object.keys(currentComponent.props);
          if (Object.keys(componentPropKeys).length !== propertyKeys.length) {
            // we have some props that aren't reflected in the docgenInfo.  Let's update the docgenInfo

            componentPropKeys.forEach(function (propKey) {
              if (!docgenInfo[editingComponent].props[propKey]) {
                _this2.props.onAddProperty(editingComponent, propKey);
              }
            });
          }

          return React__default.createElement(
            RendererContext.Provider,
            { value: _extends({}, Renderers) },
            React__default.createElement(
              'div',
              _extends({
                className: classnames('rlpEditableProps', styles$4.rlpEditableProps, className)
              }, rest),
              propertyKeys.map(function (key, idx) {
                return React__default.createElement(
                  React__default.Fragment,
                  null,
                  React__default.createElement(RendererResolver, {
                    type: filteredSchema[key].type,
                    key: key + '.' + idx,
                    name: key,
                    displayName: key,
                    value: currentComponent.props[key],
                    onChange: onChange,
                    property: filteredSchema[key]
                  }),
                  React__default.createElement('div', { className: classnames('rlpEditablePropsDivider', styles$4.rlpEditablePropsDivider) })
                );
              }),
              htmlTypes.includes(editingComponent) && React__default.createElement(AddHtmlAttribute, { pendingAttributeName: _this2.state.pendingAttributeName, onChange: _this2._onChangeProperty, pendingAttributeValue: _this2.state.pendingAttributeValue, onAddProperty: _this2._onAddProperty })
            )
          );
        }
      );
    }
  }]);
  return EditablePropsTable;
}(React.Component);

EditablePropsTable.propTypes = {
  onChange: PropTypes.func.isRequired,
  className: PropTypes.string,
  editableProperties: PropTypes.arrayOf(PropTypes.string),
  blacklistedProperties: PropTypes.arrayOf(PropTypes.string),
  onAddProperty: PropTypes.func
};

var ComponentPreview = function (_Component) {
  inherits(ComponentPreview, _Component);

  function ComponentPreview() {
    var _ref;

    var _temp, _this, _ret;

    classCallCheck(this, ComponentPreview);

    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return _ret = (_temp = (_this = possibleConstructorReturn(this, (_ref = ComponentPreview.__proto__ || Object.getPrototypeOf(ComponentPreview)).call.apply(_ref, [this].concat(args))), _this), _this.state = {
      hasError: false
    }, _temp), possibleConstructorReturn(_this, _ret);
  }

  createClass(ComponentPreview, [{
    key: 'componentDidCatch',
    value: function componentDidCatch(error, info) {
      console.error(JSON.stringify(error), info);
      this.setState({
        hasError: true
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _props = this.props,
          className = _props.className,
          rest = objectWithoutProperties(_props, ['className']);


      if (this.state.hasError) {
        return React__default.createElement(
          TextContext.Consumer,
          null,
          function (text) {
            return React__default.createElement(
              'p',
              null,
              text.componentPreviewError
            );
          }
        );
      }

      return React__default.createElement(
        SchemaContext.Consumer,
        null,
        function (_ref2) {
          var values = _ref2.values;

          return React__default.createElement(
            'div',
            _extends({
              className: classnames(className)
            }, rest),
            values
          );
        }
      );
    }
  }], [{
    key: 'getDerivedStateFromProps',
    value: function getDerivedStateFromProps(nextProps, prevState) {
      if (prevState.hasError) {
        return { hasError: false };
      }

      return prevState;
    }
  }]);
  return ComponentPreview;
}(React.Component);

ComponentPreview.propTypes = {
  className: PropTypes.string
};

var spacer = (function (times, tabStop) {
  if (times === 0) {
    return '';
  }

  return new Array(times * tabStop).fill(' ').join('');
});

/*!
 * isobject <https://github.com/jonschlinkert/isobject>
 *
 * Copyright (c) 2014-2017, Jon Schlinkert.
 * Released under the MIT License.
 */

var isobject = function isObject(val) {
  return val != null && typeof val === 'object' && Array.isArray(val) === false;
};

function isObjectObject(o) {
  return isobject(o) === true
    && Object.prototype.toString.call(o) === '[object Object]';
}

var isPlainObject = function isPlainObject(o) {
  var ctor,prot;

  if (isObjectObject(o) === false) return false;

  // If has modified constructor
  ctor = o.constructor;
  if (typeof ctor !== 'function') return false;

  // If has modified prototype
  prot = ctor.prototype;
  if (isObjectObject(prot) === false) return false;

  // If constructor does not have an Object-specific method
  if (prot.hasOwnProperty('isPrototypeOf') === false) {
    return false;
  }

  // Most likely a plain Object
  return true;
};

var isRegexp = function (re) {
	return Object.prototype.toString.call(re) === '[object RegExp]';
};

var isObj = function (x) {
	var type = typeof x;
	return x !== null && (type === 'object' || type === 'function');
};

function unwrapExports$1 (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule$1(fn, module) {
	return module = { exports: {} }, fn(module, module.exports), module.exports;
}

var lib = createCommonjsModule$1(function (module, exports) {
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = function (object) { return Object
    .getOwnPropertySymbols(object)
    .filter(function (keySymbol) { return object.propertyIsEnumerable(keySymbol); }); };

});

unwrapExports$1(lib);

var _typeof$1 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };



var getOwnEnumPropSymbols = lib.default;

var stringifyObject = function (val, opts, pad) {
	var seen = [];

	return function stringify(val, opts, pad) {
		opts = opts || {};
		opts.indent = opts.indent || '\t';
		pad = pad || '';

		var tokens = void 0;

		if (opts.inlineCharacterLimit === undefined) {
			tokens = {
				newLine: '\n',
				newLineOrSpace: '\n',
				pad: pad,
				indent: pad + opts.indent
			};
		} else {
			tokens = {
				newLine: '@@__STRINGIFY_OBJECT_NEW_LINE__@@',
				newLineOrSpace: '@@__STRINGIFY_OBJECT_NEW_LINE_OR_SPACE__@@',
				pad: '@@__STRINGIFY_OBJECT_PAD__@@',
				indent: '@@__STRINGIFY_OBJECT_INDENT__@@'
			};
		}

		var expandWhiteSpace = function expandWhiteSpace(string) {
			if (opts.inlineCharacterLimit === undefined) {
				return string;
			}

			var oneLined = string.replace(new RegExp(tokens.newLine, 'g'), '').replace(new RegExp(tokens.newLineOrSpace, 'g'), ' ').replace(new RegExp(tokens.pad + '|' + tokens.indent, 'g'), '');

			if (oneLined.length <= opts.inlineCharacterLimit) {
				return oneLined;
			}

			return string.replace(new RegExp(tokens.newLine + '|' + tokens.newLineOrSpace, 'g'), '\n').replace(new RegExp(tokens.pad, 'g'), pad).replace(new RegExp(tokens.indent, 'g'), pad + opts.indent);
		};

		if (seen.indexOf(val) !== -1) {
			return '"[Circular]"';
		}

		if (val === null || val === undefined || typeof val === 'number' || typeof val === 'boolean' || typeof val === 'function' || (typeof val === 'undefined' ? 'undefined' : _typeof$1(val)) === 'symbol' || isRegexp(val)) {
			return String(val);
		}

		if (val instanceof Date) {
			return 'new Date(\'' + val.toISOString() + '\')';
		}

		if (Array.isArray(val)) {
			if (val.length === 0) {
				return '[]';
			}

			seen.push(val);

			var ret = '[' + tokens.newLine + val.map(function (el, i) {
				var eol = val.length - 1 === i ? tokens.newLine : ',' + tokens.newLineOrSpace;
				var value = stringify(el, opts, pad + opts.indent);
				if (opts.transform) {
					value = opts.transform(val, i, value);
				}
				return tokens.indent + value + eol;
			}).join('') + tokens.pad + ']';

			seen.pop(val);

			return expandWhiteSpace(ret);
		}

		if (isObj(val)) {
			var objKeys = Object.keys(val).concat(getOwnEnumPropSymbols(val));

			if (opts.filter) {
				objKeys = objKeys.filter(function (el) {
					return opts.filter(val, el);
				});
			}

			if (objKeys.length === 0) {
				return '{}';
			}

			seen.push(val);

			var _ret = '{' + tokens.newLine + objKeys.map(function (el, i) {
				var eol = objKeys.length - 1 === i ? tokens.newLine : ',' + tokens.newLineOrSpace;
				var isSymbol = (typeof el === 'undefined' ? 'undefined' : _typeof$1(el)) === 'symbol';
				var isClassic = !isSymbol && /^[a-z$_][a-z$_0-9]*$/i.test(el);
				var key = isSymbol || isClassic ? el : stringify(el, opts);
				var value = stringify(val[el], opts, pad + opts.indent);
				if (opts.transform) {
					value = opts.transform(val, el, value);
				}
				return tokens.indent + String(key) + ': ' + value + eol;
			}).join('') + tokens.pad + '}';

			seen.pop(val);

			return expandWhiteSpace(_ret);
		}

		val = String(val).replace(/[\r\n]/g, function (x) {
			return x === '\n' ? '\\n' : '\\r';
		});

		if (opts.singleQuotes === false) {
			val = val.replace(/"/g, '\\"');
			return '"' + val + '"';
		}

		val = val.replace(/\\?'/g, '\\\'');
		return '\'' + val + '\'';
	}(val, opts, pad);
};

var _typeof$1$1 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function sortObject(value) {
  // return non-object value as is
  if (value === null || (typeof value === 'undefined' ? 'undefined' : _typeof$1$1(value)) !== 'object') {
    return value;
  }

  // return date and regexp values as is
  if (value instanceof Date || value instanceof RegExp) {
    return value;
  }

  // make a copy of array with each item passed through sortObject()
  if (Array.isArray(value)) {
    return value.map(sortObject);
  }

  // make a copy of object with key sorted
  return Object.keys(value).sort().reduce(function (result, key) {
    if (key === '_owner') {
      return result;
    }
    // eslint-disable-next-line no-param-reassign
    result[key] = sortObject(value[key]);
    return result;
  }, {});
}

/* eslint-disable no-use-before-define */

var createStringTreeNode = function createStringTreeNode(value) {
  return {
    type: 'string',
    value: value
  };
};

var createNumberTreeNode = function createNumberTreeNode(value) {
  return {
    type: 'number',
    value: value
  };
};

var createReactElementTreeNode = function createReactElementTreeNode(displayName, props, defaultProps, childrens) {
  return {
    type: 'ReactElement',
    displayName: displayName,
    props: props,
    defaultProps: defaultProps,
    childrens: childrens
  };
};

var createReactFragmentTreeNode = function createReactFragmentTreeNode(key, childrens) {
  return {
    type: 'ReactFragment',
    key: key,
    childrens: childrens
  };
};

var _typeof$2 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };


var supportFragment = Boolean(React.Fragment);

var getReactElementDisplayName = function getReactElementDisplayName(element) {
  return element.type.displayName || element.type.name || ( // function name
  typeof element.type === 'function' // function without a name, you should provide one
  ? 'No Display Name' : element.type);
};

var noChildren = function noChildren(propsValue, propName) {
  return propName !== 'children';
};

var onlyMeaningfulChildren = function onlyMeaningfulChildren(children) {
  return children !== true && children !== false && children !== null && children !== '';
};

var filterProps = function filterProps(originalProps, cb) {
  var filteredProps = {};

  Object.keys(originalProps).filter(function (key) {
    return cb(originalProps[key], key);
  }).forEach(function (key) {
    return filteredProps[key] = originalProps[key];
  });

  return filteredProps;
};

var parseReactElement = function parseReactElement(element, options) {
  var _options$displayName = options.displayName,
      displayNameFn = _options$displayName === undefined ? getReactElementDisplayName : _options$displayName;


  if (typeof element === 'string') {
    return createStringTreeNode(element);
  } else if (typeof element === 'number') {
    return createNumberTreeNode(element);
  } else if (!React__default.isValidElement(element)) {
    throw new Error('react-element-to-jsx-string: Expected a React.Element, got `' + (typeof element === 'undefined' ? 'undefined' : _typeof$2(element)) + '`');
  }

  var displayName = displayNameFn(element);

  var props = filterProps(element.props, noChildren);
  if (element.ref !== null) {
    props.ref = element.ref;
  }

  var key = element.key;
  if (typeof key === 'string' && key.search(/^\./)) {
    // React automatically add key=".X" when there are some children
    props.key = key;
  }

  var defaultProps = filterProps(element.type.defaultProps || {}, noChildren);
  var childrens = React__default.Children.toArray(element.props.children).filter(onlyMeaningfulChildren).map(function (child) {
    return parseReactElement(child, options);
  });

  if (supportFragment && element.type === React.Fragment) {
    return createReactFragmentTreeNode(key, childrens);
  }

  return createReactElementTreeNode(displayName, props, defaultProps, childrens);
};

function noRefCheck() {}

var defaultFunctionValue = function defaultFunctionValue(fn) {
  return fn.toString();
};

var formatFunction = (function (fn, options) {
  var _options$functionValu = options.functionValue,
      functionValue = _options$functionValu === undefined ? defaultFunctionValue : _options$functionValu,
      showFunctions = options.showFunctions;

  if (!showFunctions && functionValue === defaultFunctionValue) {
    return functionValue(noRefCheck);
  }

  return functionValue(fn);
});

var formatComplexDataStructure = (function (value, inline, lvl, options) {
  var normalizedValue = sortObject(value);

  var stringifiedValue = stringifyObject(normalizedValue, {
    transform: function transform(currentObj, prop, originalResult) {
      var currentValue = currentObj[prop];

      if (currentValue && React.isValidElement(currentValue)) {
        return formatTreeNode(parseReactElement(currentValue, options), true, lvl, options);
      }

      if (typeof currentValue === 'function') {
        return formatFunction(currentValue, options);
      }

      return originalResult;
    }
  });

  if (inline) {
    return stringifiedValue.replace(/\s+/g, ' ').replace(/{ /g, '{').replace(/ }/g, '}').replace(/\[ /g, '[').replace(/ ]/g, ']');
  }

  // Replace tabs with spaces, and add necessary indentation in front of each new line
  return stringifiedValue.replace(/\t/g, spacer(1, options.tabStop)).replace(/\n([^$])/g, '\n' + spacer(lvl + 1, options.tabStop) + '$1');
});

var _typeof$3 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var escape = function escape(s) {
  return s.replace(/"/g, '&quot;');
};

var formatPropValue = function formatPropValue(propValue, inline, lvl, options) {
  if (typeof propValue === 'number') {
    return '{' + String(propValue) + '}';
  }

  if (typeof propValue === 'string') {
    return '"' + escape(propValue) + '"';
  }

  // > "Symbols (new in ECMAScript 2015, not yet supported in Flow)"
  // @see: https://flow.org/en/docs/types/primitives/
  // $FlowFixMe: Flow does not support Symbol
  if ((typeof propValue === 'undefined' ? 'undefined' : _typeof$3(propValue)) === 'symbol') {
    var symbolDescription = propValue.valueOf().toString().replace(/Symbol\((.*)\)/, '$1');

    if (!symbolDescription) {
      return '{Symbol()}';
    }

    return '{Symbol(\'' + symbolDescription + '\')}';
  }

  if (typeof propValue === 'function') {
    return '{' + formatFunction(propValue, options) + '}';
  }

  if (React.isValidElement(propValue)) {
    return '{' + formatTreeNode(parseReactElement(propValue, options), true, lvl, options) + '}';
  }

  if (propValue instanceof Date) {
    return '{new Date("' + propValue.toISOString() + '")}';
  }

  if (isPlainObject(propValue) || Array.isArray(propValue)) {
    return '{' + formatComplexDataStructure(propValue, inline, lvl, options) + '}';
  }

  return '{' + String(propValue) + '}';
};

var formatProp = (function (name, hasValue, value, hasDefaultValue, defaultValue, inline, lvl, options) {
  if (!hasValue && !hasDefaultValue) {
    throw new Error('The prop "' + name + '" has no value and no default: could not be formatted');
  }

  var usedValue = hasValue ? value : defaultValue;

  var useBooleanShorthandSyntax = options.useBooleanShorthandSyntax,
      tabStop = options.tabStop;


  var formattedPropValue = formatPropValue(usedValue, inline, lvl, options);

  var attributeFormattedInline = ' ';
  var attributeFormattedMultiline = '\n' + spacer(lvl + 1, tabStop);
  var isMultilineAttribute = formattedPropValue.includes('\n');

  if (useBooleanShorthandSyntax && formattedPropValue === '{false}' && !hasDefaultValue) {
    // If a boolean is false and not different from it's default, we do not render the attribute
    attributeFormattedInline = '';
    attributeFormattedMultiline = '';
  } else if (useBooleanShorthandSyntax && formattedPropValue === '{true}') {
    attributeFormattedInline += '' + name;
    attributeFormattedMultiline += '' + name;
  } else {
    attributeFormattedInline += name + '=' + formattedPropValue;
    attributeFormattedMultiline += name + '=' + formattedPropValue;
  }

  return {
    attributeFormattedInline: attributeFormattedInline,
    attributeFormattedMultiline: attributeFormattedMultiline,
    isMultilineAttribute: isMultilineAttribute
  };
});

var mergeSiblingPlainStringChildrenReducer = (function (previousNodes, currentNode) {
  var nodes = previousNodes.slice(0, previousNodes.length > 0 ? previousNodes.length - 1 : 0);
  var previousNode = previousNodes[previousNodes.length - 1];

  if (previousNode && (currentNode.type === 'string' || currentNode.type === 'number') && (previousNode.type === 'string' || previousNode.type === 'number')) {
    nodes.push(createStringTreeNode(String(previousNode.value) + String(currentNode.value)));
  } else {
    if (previousNode) {
      nodes.push(previousNode);
    }

    nodes.push(currentNode);
  }

  return nodes;
});

var propNameSorter = (function (sortProps) {
  return function (a, b) {
    if (a === b) {
      return 0;
    }

    if (['key', 'ref'].includes(a)) {
      return -1;
    } else if (['key', 'ref'].includes(b)) {
      return 1;
    }

    if (!sortProps) {
      return 0;
    }

    return a < b ? -1 : 1;
  };
});

var compensateMultilineStringElementIndentation = function compensateMultilineStringElementIndentation(element, formattedElement, inline, lvl, options) {
  var tabStop = options.tabStop;


  if (element.type === 'string') {
    return formattedElement.split('\n').map(function (line, offset) {
      if (offset === 0) {
        return line;
      }

      return '' + spacer(lvl, tabStop) + line;
    }).join('\n');
  }

  return formattedElement;
};

var formatOneChildren = function formatOneChildren(inline, lvl, options) {
  return function (element) {
    return compensateMultilineStringElementIndentation(element, formatTreeNode(element, inline, lvl, options), inline, lvl, options);
  };
};

var onlyPropsWithOriginalValue = function onlyPropsWithOriginalValue(defaultProps, props) {
  return function (propName) {
    var haveDefaultValue = Object.keys(defaultProps).includes(propName);
    return !haveDefaultValue || haveDefaultValue && defaultProps[propName] !== props[propName];
  };
};

var isInlineAttributeTooLong = function isInlineAttributeTooLong(attributes, inlineAttributeString, lvl, tabStop, maxInlineAttributesLineLength) {
  if (!maxInlineAttributesLineLength) {
    return attributes.length > 1;
  }

  return spacer(lvl, tabStop).length + inlineAttributeString.length > maxInlineAttributesLineLength;
};

var shouldRenderMultilineAttr = function shouldRenderMultilineAttr(attributes, inlineAttributeString, containsMultilineAttr, inline, lvl, tabStop, maxInlineAttributesLineLength) {
  return (isInlineAttributeTooLong(attributes, inlineAttributeString, lvl, tabStop, maxInlineAttributesLineLength) || containsMultilineAttr) && !inline;
};

var formatReactElementNode = (function (node, inline, lvl, options) {
  var type = node.type,
      _node$displayName = node.displayName,
      displayName = _node$displayName === undefined ? '' : _node$displayName,
      childrens = node.childrens,
      _node$props = node.props,
      props = _node$props === undefined ? {} : _node$props,
      _node$defaultProps = node.defaultProps,
      defaultProps = _node$defaultProps === undefined ? {} : _node$defaultProps;


  if (type !== 'ReactElement') {
    throw new Error('The "formatReactElementNode" function could only format node of type "ReactElement". Given:  ' + type);
  }

  var filterProps = options.filterProps,
      maxInlineAttributesLineLength = options.maxInlineAttributesLineLength,
      showDefaultProps = options.showDefaultProps,
      sortProps = options.sortProps,
      tabStop = options.tabStop;


  var out = '<' + displayName;

  var outInlineAttr = out;
  var outMultilineAttr = out;
  var containsMultilineAttr = false;

  var visibleAttributeNames = [];

  Object.keys(props).filter(function (propName) {
    return filterProps.indexOf(propName) === -1;
  }).filter(onlyPropsWithOriginalValue(defaultProps, props)).forEach(function (propName) {
    return visibleAttributeNames.push(propName);
  });

  Object.keys(defaultProps).filter(function (defaultPropName) {
    return filterProps.indexOf(defaultPropName) === -1;
  }).filter(function () {
    return showDefaultProps;
  }).filter(function (defaultPropName) {
    return !visibleAttributeNames.includes(defaultPropName);
  }).forEach(function (defaultPropName) {
    return visibleAttributeNames.push(defaultPropName);
  });

  var attributes = visibleAttributeNames.sort(propNameSorter(sortProps));

  attributes.forEach(function (attributeName) {
    var _formatProp = formatProp(attributeName, Object.keys(props).includes(attributeName), props[attributeName], Object.keys(defaultProps).includes(attributeName), defaultProps[attributeName], inline, lvl, options),
        attributeFormattedInline = _formatProp.attributeFormattedInline,
        attributeFormattedMultiline = _formatProp.attributeFormattedMultiline,
        isMultilineAttribute = _formatProp.isMultilineAttribute;

    if (isMultilineAttribute) {
      containsMultilineAttr = true;
    }

    outInlineAttr += attributeFormattedInline;
    outMultilineAttr += attributeFormattedMultiline;
  });

  outMultilineAttr += '\n' + spacer(lvl, tabStop);

  if (shouldRenderMultilineAttr(attributes, outInlineAttr, containsMultilineAttr, inline, lvl, tabStop, maxInlineAttributesLineLength)) {
    out = outMultilineAttr;
  } else {
    out = outInlineAttr;
  }

  if (childrens && childrens.length > 0) {
    var newLvl = lvl + 1;

    out += '>';

    if (!inline) {
      out += '\n';
      out += spacer(newLvl, tabStop);
    }

    out += childrens.reduce(mergeSiblingPlainStringChildrenReducer, []).map(formatOneChildren(inline, newLvl, options)).join(!inline ? '\n' + spacer(newLvl, tabStop) : '');

    if (!inline) {
      out += '\n';
      out += spacer(newLvl - 1, tabStop);
    }
    out += '</' + displayName + '>';
  } else {
    if (!isInlineAttributeTooLong(attributes, outInlineAttr, lvl, tabStop, maxInlineAttributesLineLength)) {
      out += ' ';
    }

    out += '/>';
  }

  return out;
});

var REACT_FRAGMENT_TAG_NAME_SHORT_SYNTAX = '';
var REACT_FRAGMENT_TAG_NAME_EXPLICIT_SYNTAX = 'React.Fragment';

var toReactElementTreeNode = function toReactElementTreeNode(displayName, key, childrens) {
  var props = {};
  if (key) {
    props = { key: key };
  }

  return {
    type: 'ReactElement',
    displayName: displayName,
    props: props,
    defaultProps: {},
    childrens: childrens
  };
};

var isKeyedFragment = function isKeyedFragment(_ref) {
  var key = _ref.key;
  return Boolean(key);
};
var hasNoChildren = function hasNoChildren(_ref2) {
  var childrens = _ref2.childrens;
  return childrens.length === 0;
};

var formatReactFragmentNode = (function (node, inline, lvl, options) {
  var type = node.type,
      key = node.key,
      childrens = node.childrens;


  if (type !== 'ReactFragment') {
    throw new Error('The "formatReactFragmentNode" function could only format node of type "ReactFragment". Given: ' + type);
  }

  var useFragmentShortSyntax = options.useFragmentShortSyntax;


  var displayName = void 0;
  if (useFragmentShortSyntax) {
    if (hasNoChildren(node) || isKeyedFragment(node)) {
      displayName = REACT_FRAGMENT_TAG_NAME_EXPLICIT_SYNTAX;
    } else {
      displayName = REACT_FRAGMENT_TAG_NAME_SHORT_SYNTAX;
    }
  } else {
    displayName = REACT_FRAGMENT_TAG_NAME_EXPLICIT_SYNTAX;
  }

  return formatReactElementNode(toReactElementTreeNode(displayName, key, childrens), inline, lvl, options);
});

var jsxStopChars = ['<', '>', '{', '}'];
var shouldBeEscaped = function shouldBeEscaped(s) {
  return jsxStopChars.some(function (jsxStopChar) {
    return s.includes(jsxStopChar);
  });
};

var escape$1 = function escape(s) {
  if (!shouldBeEscaped(s)) {
    return s;
  }

  return '{`' + s + '`}';
};

var preserveTrailingSpace = function preserveTrailingSpace(s) {
  var result = s;
  if (result.endsWith(' ')) {
    result = result.replace(/^(\S*)(\s*)$/, "$1{'$2'}");
  }

  if (result.startsWith(' ')) {
    result = result.replace(/^(\s*)(\S*)$/, "{'$1'}$2");
  }

  return result;
};

var formatTreeNode = (function (node, inline, lvl, options) {
  if (node.type === 'number') {
    return String(node.value);
  }

  if (node.type === 'string') {
    return node.value ? '' + preserveTrailingSpace(escape$1(String(node.value))) : '';
  }

  if (node.type === 'ReactElement') {
    return formatReactElementNode(node, inline, lvl, options);
  }

  if (node.type === 'ReactFragment') {
    return formatReactFragmentNode(node, inline, lvl, options);
  }

  throw new TypeError('Unknow format type "' + node.type + '"');
});

var formatTree = (function (node, options) {
  return formatTreeNode(node, false, 0, options);
});

var reactElementToJsxString = function reactElementToJsxString(element) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$filterProps = _ref.filterProps,
      filterProps = _ref$filterProps === undefined ? [] : _ref$filterProps,
      _ref$showDefaultProps = _ref.showDefaultProps,
      showDefaultProps = _ref$showDefaultProps === undefined ? true : _ref$showDefaultProps,
      _ref$showFunctions = _ref.showFunctions,
      showFunctions = _ref$showFunctions === undefined ? false : _ref$showFunctions,
      functionValue = _ref.functionValue,
      _ref$tabStop = _ref.tabStop,
      tabStop = _ref$tabStop === undefined ? 2 : _ref$tabStop,
      _ref$useBooleanShorth = _ref.useBooleanShorthandSyntax,
      useBooleanShorthandSyntax = _ref$useBooleanShorth === undefined ? true : _ref$useBooleanShorth,
      _ref$useFragmentShort = _ref.useFragmentShortSyntax,
      useFragmentShortSyntax = _ref$useFragmentShort === undefined ? true : _ref$useFragmentShort,
      _ref$sortProps = _ref.sortProps,
      sortProps = _ref$sortProps === undefined ? true : _ref$sortProps,
      maxInlineAttributesLineLength = _ref.maxInlineAttributesLineLength,
      displayName = _ref.displayName;

  if (!element) {
    throw new Error('react-element-to-jsx-string: Expected a ReactElement');
  }

  var options = {
    filterProps: filterProps,
    showDefaultProps: showDefaultProps,
    showFunctions: showFunctions,
    functionValue: functionValue,
    tabStop: tabStop,
    useBooleanShorthandSyntax: useBooleanShorthandSyntax,
    useFragmentShortSyntax: useFragmentShortSyntax,
    sortProps: sortProps,
    maxInlineAttributesLineLength: maxInlineAttributesLineLength,
    displayName: displayName
  };

  return formatTree(parseReactElement(element, options), options);
};

var css$6 = ".styles_codeRoot__12-9M code {\n  padding: 12px;\n}\n";
var styles$5 = { "codeRoot": "styles_codeRoot__12-9M" };
styleInject(css$6);

/* global Prism */

var ComponentMarkup = function (_Component) {
  inherits(ComponentMarkup, _Component);

  function ComponentMarkup() {
    classCallCheck(this, ComponentMarkup);
    return possibleConstructorReturn(this, (ComponentMarkup.__proto__ || Object.getPrototypeOf(ComponentMarkup)).apply(this, arguments));
  }

  createClass(ComponentMarkup, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      // we don't include prism, but consumers can provide prism.js and prism.css and this will apply
      if (!Prism) return;

      this.node.querySelectorAll('code').forEach(function (block) {
        Prism.highlightElement(block);
      });
    }
  }, {
    key: 'componentDidUpdate',
    value: function componentDidUpdate() {
      if (!Prism) return;

      this.node.querySelectorAll('code').forEach(function (block) {
        Prism.highlightElement(block);
      });
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          className = _props.className,
          rest = objectWithoutProperties(_props, ['className']);


      return React__default.createElement(
        SchemaContext.Consumer,
        null,
        function (_ref) {
          var values = _ref.values;

          return React__default.createElement(
            'div',
            _extends({
              ref: function ref(_ref2) {
                _this2.node = _ref2;
              },
              className: classnames('codeRoot', styles$5.codeRoot, className)
            }, rest),
            React__default.createElement(
              'pre',
              null,
              React__default.createElement(
                'code',
                { className: 'language-jsx' },
                reactElementToJsxString(values)
              )
            )
          );
        }
      );
    }
  }]);
  return ComponentMarkup;
}(React.Component);

ComponentMarkup.propTypes = {
  className: PropTypes.string
};

var css$7 = ".styles_rlpTreeView__2E3lL ul,\n.styles_rlpTreeView__2E3lL li {\n  margin: 0px;\n  padding: 0px;\n  list-style-type: none;\n}\n\n.styles_rlpTreeView__2E3lL .styles_rlpTreeViewSelected__1aOJh {\n  font-weight: bold;\n}\n\n.styles_rlpTreeView__2E3lL ul { padding-left: 0.5em; }\n.styles_rlpTreeView__2E3lL li  { padding-left: 0.5em;\n  border: 1px dotted black;\n  border-width: 0 0 1px 1px;\n}\n.styles_rlpTreeView__2E3lL li.styles_rlpTreeViewListContainer__2_8YN { border-bottom: 0px; }\n\n.styles_rlpTreeView__2E3lL li p { margin: 0;\n  background: white;\n  position: relative;\n  top: 0.5em;\n}\n.styles_rlpTreeView__2E3lL li ul {\n  border-top: 1px dotted black;\n  margin-left: -0.5em;\n  padding-left: 1em;\n}\n.styles_rlpTreeView__2E3lL ul li:last-child ul {\n  border-left: 1px solid white;\n  margin-left: -17px;\n}\n\n.styles_rlpTreeViewPropContainer__22_kv {\n  padding-left: 0.5em;\n}\n";
var styles$6 = { "rlpTreeView": "styles_rlpTreeView__2E3lL", "rlpTreeViewSelected": "styles_rlpTreeViewSelected__1aOJh", "rlpTreeViewListContainer": "styles_rlpTreeViewListContainer__2_8YN", "rlpTreeViewPropContainer": "styles_rlpTreeViewPropContainer__22_kv" };
styleInject(css$7);

var arePathsEquivalent = function arePathsEquivalent(a, b) {
  if (a.length !== b.length) return false;

  for (var i = 0; i < a.length; i++) {
    if (a[i] !== b[i]) return false;
  }

  return true;
};

var RenderComponent = function RenderComponent(_ref) {
  var values = _ref.values,
      componentDisplayName = _ref.componentDisplayName,
      onChangeComponent = _ref.onChangeComponent,
      componentPath = _ref.componentPath;

  return React__default.createElement(
    SchemaContext.Consumer,
    null,
    function (_ref2) {
      var docgenInfo = _ref2.docgenInfo,
          editingComponentPath = _ref2.editingComponentPath;

      var isSelected = arePathsEquivalent(editingComponentPath, componentPath);
      return React__default.createElement(
        React__default.Fragment,
        null,
        React__default.createElement(
          'li',
          null,
          React__default.createElement(
            'p',
            null,
            React__default.createElement(
              'a',
              { href: 'javascript:void(0)', className: isSelected ? classnames('rlpTreeViewSelected', styles$6.rlpTreeViewSelected) : '', onClick: function onClick() {
                  return onChangeComponent(componentDisplayName, componentPath);
                } },
              componentDisplayName
            )
          )
        ),
        typeof values !== 'string' && docgenInfo[componentDisplayName] && findComponentProperties(docgenInfo[componentDisplayName].props).map(function (nodeProp, nodeIdx) {
          if (values && values.props[nodeProp]) {
            if (typeof values.props[nodeProp] === 'string') return null;

            if (Array.isArray(values.props[nodeProp]) && values.props[nodeProp].length === 0) return null;

            if (Object.keys(values.props[nodeProp]).length === 0) return null;
          } else {
            return null;
          }

          return React__default.createElement(
            'li',
            { className: classnames('rlpTreeViewListContainer', styles$6.rlpTreeViewListContainer), key: componentDisplayName + '.' + nodeProp + '.' + nodeIdx },
            React__default.createElement(
              'div',
              { className: classnames('rlpTreeViewPropContainer', styles$6.rlpTreeViewPropContainer) },
              React__default.createElement(
                'p',
                null,
                nodeProp
              ),
              React__default.createElement(
                'ul',
                null,
                Array.isArray(values.props[nodeProp]) && values.props[nodeProp].map(function (child, idx) {
                  var newPath = [].concat(toConsumableArray(componentPath), ['props', nodeProp, idx]);
                  return React__default.createElement(RenderComponent, { key: child.type + '-' + idx, componentPath: newPath, values: values.props[nodeProp][idx], componentDisplayName: getDisplayName(child), onChangeComponent: onChangeComponent });
                }),
                !Array.isArray(values.props[nodeProp]) && values.props[nodeProp] && React__default.createElement(RenderComponent, { componentPath: [].concat(toConsumableArray(componentPath), ['props', nodeProp]), values: values.props[nodeProp], componentDisplayName: getDisplayName(values.props[nodeProp]), onChangeComponent: onChangeComponent })
              )
            )
          );
        })
      );
    }
  );
};

RenderComponent.propTypes = {
  values: PropTypes.oneOfType([PropTypes.object, PropTypes.string]).isRequired,
  componentDisplayName: PropTypes.string.isRequired,
  onChangeComponent: PropTypes.func.isRequired,
  componentPath: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.number])).isRequired
};

var TreeView = function (_React$Component) {
  inherits(TreeView, _React$Component);

  function TreeView() {
    classCallCheck(this, TreeView);
    return possibleConstructorReturn(this, (TreeView.__proto__ || Object.getPrototypeOf(TreeView)).apply(this, arguments));
  }

  createClass(TreeView, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          of = _props.of,
          className = _props.className,
          onChangeComponent = _props.onChangeComponent,
          rest = objectWithoutProperties(_props, ['of', 'className', 'onChangeComponent']);


      return React__default.createElement(
        SchemaContext.Consumer,
        null,
        function (_ref3) {
          var values = _ref3.values,
              rootComponentDisplayName = _ref3.rootComponentDisplayName,
              docgenInfo = _ref3.docgenInfo;

          return React__default.createElement(
            'div',
            _extends({ className: classnames('rlpTreeView', styles$6.rlpTreeView, className) }, rest),
            React__default.createElement(
              'ul',
              null,
              React__default.createElement(RenderComponent, { values: values, componentPath: [], componentDisplayName: rootComponentDisplayName, onChangeComponent: onChangeComponent })
            )
          );
        }
      );
    }
  }]);
  return TreeView;
}(React__default.Component);

TreeView.propTypes = {
  of: PropTypes.func.isRequired,
  className: PropTypes.string,
  onChangeComponent: PropTypes.func.isRequired
};

var uiElements = {
  SaveButton: function SaveButton(_ref) {
    var onClick = _ref.onClick,
        className = _ref.className;
    return React__default.createElement(
      'button',
      { type: 'button', className: className, onClick: onClick, 'aria-label': 'Save Item' },
      React__default.createElement(MdSave, null)
    );
  },
  AddButton: function AddButton(_ref2) {
    var onClick = _ref2.onClick,
        className = _ref2.className;
    return React__default.createElement(
      'button',
      { type: 'button', className: className, onClick: onClick, 'aria-label': 'Add Item' },
      React__default.createElement(MdAddCircle, null)
    );
  },
  DeleteButton: function DeleteButton(_ref3) {
    var onClick = _ref3.onClick,
        className = _ref3.className;
    return React__default.createElement(
      'button',
      { type: 'button', className: className, onClick: onClick, 'aria-label': 'Delete Item' },
      React__default.createElement(MdDelete, null)
    );
  },
  ResetButton: function ResetButton(_ref4) {
    var onClick = _ref4.onClick,
        className = _ref4.className;
    return React__default.createElement(
      TextContext.Consumer,
      null,
      function (text$$1) {
        return React__default.createElement(
          'button',
          { type: 'button', className: className, onClick: onClick, 'aria-label': 'Reset Props' },
          text$$1.reset
        );
      }
    );
  }
};

uiElements.SaveButton.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

uiElements.AddButton.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

uiElements.DeleteButton.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

uiElements.ResetButton.propTypes = {
  onClick: PropTypes.func.isRequired,
  className: PropTypes.string
};

var DEFAULT_HTML_TYPES = ['p', 'a', 'em', 'span', 'strong', 'div', 'svg', 'path', 'ul', 'li', 'b', 'ol', 'blockquote', 'cite', 'pre', 'code', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'img', 'caption', 'table', 'tbody', 'thead', 'tr', 'td', 'th', 'tfoot'];
var buildComponentTitle = function buildComponentTitle(title, additionalTitleText) {
  if (!additionalTitleText) return title;

  return title + ' - ' + additionalTitleText;
};

var initialize = function initialize(_ref5) {
  var of = _ref5.of,
      docgenInfo = _ref5.docgenInfo,
      additionalTitleText = _ref5.additionalTitleText,
      availableTypes = _ref5.availableTypes,
      initialComponent = _ref5.initialComponent,
      _ref5$addButtonCompon = _ref5.addButtonComponent,
      addButtonComponent = _ref5$addButtonCompon === undefined ? uiElements.AddButton : _ref5$addButtonCompon,
      _ref5$deleteButtonCom = _ref5.deleteButtonComponent,
      deleteButtonComponent = _ref5$deleteButtonCom === undefined ? uiElements.DeleteButton : _ref5$deleteButtonCom,
      _ref5$saveButtonCompo = _ref5.saveButtonComponent,
      saveButtonComponent = _ref5$saveButtonCompo === undefined ? uiElements.SaveButton : _ref5$saveButtonCompo,
      _ref5$resetButtonComp = _ref5.resetButtonComponent,
      resetButtonComponent = _ref5$resetButtonComp === undefined ? uiElements.ResetButton : _ref5$resetButtonComp,
      _ref5$overrideText = _ref5.overrideText,
      overrideText = _ref5$overrideText === undefined ? {} : _ref5$overrideText;

  var htmlTypes = [].concat(DEFAULT_HTML_TYPES);
  var allDocGenInfo = [];
  var info = docgenInfo || of.__docgenInfo;
  if (!info) {
    throw new Error('ReactLiveProps must be given docgenInfo or a component annotated with __docgenInfo');
  }

  allDocGenInfo.push(_extends({
    props: {}
  }, info));

  allDocGenInfo.push({
    description: 'React.Fragment',
    methods: [],
    props: {
      children: {
        description: '',
        required: false,
        type: {
          name: 'arrayOf',
          value: { name: 'node' }
        }
      }
    },
    displayName: getDisplayName('React.Fragment')
  });

  var availableChildren = [{ name: 'React.Fragment' }].concat(DEFAULT_HTML_TYPES);
  if (availableTypes) {
    availableTypes.forEach(function (child) {
      if (availableChildren.includes(child)) return;

      availableChildren.push(child);
    });

    var typeInfo = DEFAULT_HTML_TYPES.concat(availableTypes).map(function (type) {
      if (typeof type === 'string') {
        if (!htmlTypes.includes(type)) {
          htmlTypes.push(type);
        }

        return {
          description: '',
          methods: [],
          props: {
            children: {
              description: '',
              required: false,
              type: {
                name: 'arrayOf',
                value: { name: 'node' }
              }
            }
          },
          displayName: type
        };
      }

      if (!type.__docgenInfo) {
        console.error('Docgen info missing for type', type);
      }

      return type.__docgenInfo;
    });

    var filteredTypes = typeInfo.filter(function (type) {
      return type !== null;
    }).map(function (type) {
      return _extends({
        props: {}
      }, type);
    });
    if (filteredTypes.filter(function (type) {
      return typeof type === 'undefined';
    }).length > 0) {
      throw new Error('ReactLiveProps must be given docgenInfo or a component annotated with __docgenInfo');
    }
    allDocGenInfo.push.apply(allDocGenInfo, toConsumableArray(filteredTypes));
  } else {
    var _typeInfo = DEFAULT_HTML_TYPES.map(function (type) {
      if (typeof type === 'string') {
        if (!htmlTypes.includes(type)) {
          htmlTypes.push(type);
        }

        return {
          description: '',
          methods: [],
          props: {
            children: {
              description: '',
              required: false,
              type: {
                name: 'arrayOf',
                value: { name: 'node' }
              }
            }
          },
          displayName: type
        };
      }

      if (!type.__docgenInfo) {
        console.error('Docgen info missing for type', type);
      }

      return type.__docgenInfo;
    });

    var _filteredTypes = _typeInfo.filter(function (type) {
      return type !== null;
    }).map(function (type) {
      return _extends({
        props: {}
      }, type);
    });
    if (_filteredTypes.filter(function (type) {
      return typeof type === 'undefined';
    }).length > 0) {
      throw new Error('ReactLiveProps must be given docgenInfo or a component annotated with __docgenInfo');
    }
    allDocGenInfo.push.apply(allDocGenInfo, toConsumableArray(_filteredTypes));
  }

  try {
    var _docgenInfo = {};
    allDocGenInfo.forEach(function (typeInfo) {
      try {
        _docgenInfo[getDisplayName(typeInfo)] = typeInfo;
      } catch (e) {
        console.error('Error parsing schema for type', typeInfo, e);
        throw new Error('Error parsing schema for type ', typeInfo.displayName);
      }
    });

    var safeDisplayName = getDisplayName(info);

    var values = initialComponent || React__default.createElement(of);

    var configuredUiElements = {
      AddButton: addButtonComponent,
      SaveButton: saveButtonComponent,
      DeleteButton: deleteButtonComponent,
      ResetButton: resetButtonComponent
    };

    return {
      title: buildComponentTitle(safeDisplayName, additionalTitleText),
      values: values,
      rootComponentDisplayName: safeDisplayName,
      editingComponent: safeDisplayName,
      editingComponentPath: [],
      htmlTypes: htmlTypes,
      availableTypes: availableChildren,
      docgenInfo: _docgenInfo,
      configuredUiElements: configuredUiElements,
      text: _extends({}, text, overrideText)
    };
  } catch (err) {
    console.error('ReactLiveProps error initializing', err);
    return {
      errorMessage: err.message
    };
  }
};

var css$8 = ".styles_rlpContainer__1_UA4 {\n  border: 2px solid #000;\n  padding: 12px;\n  background: white;\n}\n\n.styles_rlpContainerTitle__2jEL1 {\n  padding: 12px;\n  border-bottom: 1px solid #e5e5e5;\n}\n\n.styles_rlpContainerSubTitle__YbD-r {\n  margin: 12px 0px;\n}\n\n.styles_rlpContainerSubTitle__YbD-r > button {\n  margin-left: 12px;\n}\n\n.styles_rlpSection__1jrLD {\n  margin-top: 12px;\n  margin-bottom: 12px;\n}\n\n.styles_rlpComponentPreview__2EFXk, .styles_rlpEditablePropsTable__20Xq7, .styles_rlpPropsTableMain__2YTT5, .styles_rlpCustomComponentMarkup__I07SC, .styles_rlpComponentMarkup__2imbH {\n  padding: 12px;\n  border: 2px solid #e5e5e5;\n}\n\n.styles_rlpEditablePropsTableTreeView__2MrVU {\n  max-height: 400px;\n  overflow: auto;\n  min-width: 150px;\n  padding-right: 16px;\n}\n\n.styles_rlpFlexContainer__2kESD {\n  display: flex;\n}\n\n.styles_rlpEditablePropsTableSpacer__uXEjm {\n  width: 4px;\n  margin: 0px 10px;\n  background-color: #e5e5e5;\n}\n\n.styles_rlpEditablePropsTableMain__27jom {\n  flex-grow: 1;\n  max-height: 400px;\n  overflow-y: auto;\n}\n\n.styles_rlpContainerTitle__2jEL1 button {\n  float: right;\n  background-color: transparent;\n  border: none;\n}\n\n.styles_rlpComponentMarkup__2imbH code {\n  font-size: 14px;\n}\n";
var styles$7 = { "rlpContainer": "styles_rlpContainer__1_UA4", "rlpContainerTitle": "styles_rlpContainerTitle__2jEL1", "rlpContainerSubTitle": "styles_rlpContainerSubTitle__YbD-r", "rlpSection": "styles_rlpSection__1jrLD", "rlpComponentPreview": "styles_rlpComponentPreview__2EFXk", "rlpEditablePropsTable": "styles_rlpEditablePropsTable__20Xq7", "rlpPropsTableMain": "styles_rlpPropsTableMain__2YTT5", "rlpCustomComponentMarkup": "styles_rlpCustomComponentMarkup__I07SC", "rlpComponentMarkup": "styles_rlpComponentMarkup__2imbH", "rlpEditablePropsTableTreeView": "styles_rlpEditablePropsTableTreeView__2MrVU", "rlpFlexContainer": "styles_rlpFlexContainer__2kESD", "rlpEditablePropsTableSpacer": "styles_rlpEditablePropsTableSpacer__uXEjm", "rlpEditablePropsTableMain": "styles_rlpEditablePropsTableMain__27jom" };
styleInject(css$8);

var ReactLiveProps = function (_Component) {
  inherits(ReactLiveProps, _Component);

  function ReactLiveProps(props) {
    classCallCheck(this, ReactLiveProps);

    var _this = possibleConstructorReturn(this, (ReactLiveProps.__proto__ || Object.getPrototypeOf(ReactLiveProps)).call(this, props));

    _this.onToggleExpandCollapse = function (collapsed) {
      _this.setState({
        collapsed: collapsed
      });
    };

    _this._editComponent = function (rootComponentDisplayName, componentPath) {
      _this.setState({
        editingComponent: rootComponentDisplayName,
        editingComponentPath: componentPath
      });
    };

    _this._onChange = function (values) {
      _this.setState({
        values: values
      });
    };

    _this._onAddProperty = function (editingComponent, attributeName) {
      var newDocgenInfo = _extends({}, _this.state.docgenInfo);

      newDocgenInfo[editingComponent].props = _extends({}, newDocgenInfo[editingComponent].props, defineProperty({}, attributeName, {
        description: _this.state.text.htmlAttributeDescription,
        required: false,
        type: {
          name: 'any'
        }
      }));
      _this.setState({
        docgenInfo: newDocgenInfo
      });
    };

    _this._reset = function () {
      _this.setState(initialize(_this.props));
    };

    _this.state = {
      title: null,
      values: null,
      collapsed: _this.props.initialCollapsed || false,
      errorMessage: null
    };
    return _this;
  }

  createClass(ReactLiveProps, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.setState(initialize(this.props));
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          of = _props.of,
          docgenInfo = _props.docgenInfo,
          className = _props.className,
          editableProperties = _props.editableProperties,
          additionalTitleText = _props.additionalTitleText,
          hideComponentMarkup = _props.hideComponentMarkup,
          blacklistedProperties = _props.blacklistedProperties,
          hideComponentPreview = _props.hideComponentPreview,
          CustomComponentMarkup = _props.customComponentMarkup,
          initialComponent = _props.initialComponent,
          availableTypes = _props.availableTypes,
          initialCollapsed = _props.initialCollapsed,
          hidePropsTable = _props.hidePropsTable,
          hideHeader = _props.hideHeader,
          addButtonComponent = _props.addButtonComponent,
          deleteButtonComponent = _props.deleteButtonComponent,
          saveButtonComponent = _props.saveButtonComponent,
          resetButtonComponent = _props.resetButtonComponent,
          rest = objectWithoutProperties(_props, ['of', 'docgenInfo', 'className', 'editableProperties', 'additionalTitleText', 'hideComponentMarkup', 'blacklistedProperties', 'hideComponentPreview', 'customComponentMarkup', 'initialComponent', 'availableTypes', 'initialCollapsed', 'hidePropsTable', 'hideHeader', 'addButtonComponent', 'deleteButtonComponent', 'saveButtonComponent', 'resetButtonComponent']);
      var _state = this.state,
          title = _state.title,
          values = _state.values,
          rootComponentDisplayName = _state.rootComponentDisplayName,
          editingComponent = _state.editingComponent,
          htmlTypes = _state.htmlTypes,
          editingComponentPath = _state.editingComponentPath,
          availableChildTypes = _state.availableTypes,
          allDocGenInfo = _state.docgenInfo,
          configuredUiElements = _state.configuredUiElements;


      if (this.state.errorMessage) {
        return React__default.createElement(
          'p',
          { className: classnames('rlpErrorMessage', styles$7.rlpErrorMessage) },
          this.state.errorMessage
        );
      }

      if (!values) {
        return null;
      }

      var ResetButton = configuredUiElements.ResetButton;

      return React__default.createElement(
        TextContext.Provider,
        { value: this.state.text },
        React__default.createElement(
          UIContext.Provider,
          { value: configuredUiElements },
          React__default.createElement(
            SchemaContext.Provider,
            { value: { values: values, rootComponentDisplayName: rootComponentDisplayName, editingComponent: editingComponent, editingComponentPath: editingComponentPath, htmlTypes: htmlTypes, availableTypes: availableChildTypes, docgenInfo: allDocGenInfo } },
            React__default.createElement(
              'div',
              _extends({
                className: classnames('rlpContainer', styles$7.rlpContainer, className)
              }, rest),
              !hideHeader && React__default.createElement(
                'h2',
                {
                  className: classnames('rlpContainerTitle', styles$7.rlpContainerTitle),
                  role: 'button',
                  'aria-label': this.state.text.expandCollapseAriaLabel,
                  'aria-expanded': !this.state.collapsed,
                  onClick: function onClick() {
                    return _this2.onToggleExpandCollapse(!_this2.state.collapsed);
                  }
                },
                title,
                this.state.collapsed && React__default.createElement(Expand, { onClick: function onClick() {
                    return _this2.onToggleExpandCollapse(!_this2.state.collapsed);
                  } }),
                !this.state.collapsed && React__default.createElement(Collapse, { onClick: function onClick() {
                    return _this2.onToggleExpandCollapse(!_this2.state.collapsed);
                  } })
              ),
              !this.state.collapsed && React__default.createElement(
                React__default.Fragment,
                null,
                !hidePropsTable && React__default.createElement(
                  'div',
                  { className: classnames('rlpSection', 'rlpPropsTableMain', styles$7.rlpSection, styles$7.rlpPropsTableMain) },
                  React__default.createElement(
                    'h6',
                    { className: classnames('rlpContainerSubTitle', styles$7.rlpContainerSubTitle) },
                    this.state.text.propTableHeader
                  ),
                  React__default.createElement(PropsTable, null)
                ),
                !hideComponentPreview && React__default.createElement(
                  'div',
                  { className: classnames('rlpSection', 'rlpComponentPreview', styles$7.rlpSection, styles$7.rlpComponentPreview) },
                  React__default.createElement(
                    'h6',
                    { className: classnames('rlpContainerSubTitle', styles$7.rlpContainerSubTitle) },
                    this.state.text.componentPreviewHeader
                  ),
                  React__default.createElement(ComponentPreview, null)
                ),
                React__default.createElement(
                  'div',
                  { className: classnames('rlpSection', 'rlpEditablePropsTable', styles$7.rlpSection, styles$7.rlpEditablePropsTable) },
                  React__default.createElement(
                    'h6',
                    { className: classnames('rlpContainerSubTitle', styles$7.rlpContainerSubTitle) },
                    this.state.text.propsEditorHeader,
                    React__default.createElement(ResetButton, { onClick: this._reset })
                  ),
                  React__default.createElement(
                    'div',
                    { className: classnames('rlpFlexContainer', styles$7.rlpFlexContainer) },
                    findComponentProperties(allDocGenInfo[rootComponentDisplayName].props).length > 0 && React__default.createElement(
                      React__default.Fragment,
                      null,
                      React__default.createElement(
                        'div',
                        { className: classnames('rlpEditablePropsTableTreeView', styles$7.rlpEditablePropsTableTreeView) },
                        React__default.createElement(TreeView, {
                          of: of,
                          onChangeComponent: this._editComponent
                        })
                      ),
                      React__default.createElement('div', { className: classnames('rlpEditablePropsTableSpacer', styles$7.rlpEditablePropsTableSpacer) })
                    ),
                    React__default.createElement(
                      'div',
                      { className: classnames('rlpEditablePropsTableMain', styles$7.rlpEditablePropsTableMain) },
                      React__default.createElement(EditablePropsTable, {
                        editableProperties: editableProperties,
                        blacklistedProperties: blacklistedProperties,
                        onChange: this._onChange,
                        onAddProperty: this._onAddProperty,
                        onReset: this._reset
                      })
                    )
                  )
                ),
                !hideComponentMarkup && React__default.createElement(
                  'div',
                  { className: classnames('rlpSection', 'rlpComponentMarkup', styles$7.rlpSection, styles$7.rlpComponentMarkup) },
                  React__default.createElement(
                    'h6',
                    { className: classnames('rlpContainerSubTitle', styles$7.rlpContainerSubTitle) },
                    this.state.text.componentMarkupHeader
                  ),
                  React__default.createElement(ComponentMarkup, null)
                ),
                CustomComponentMarkup && React__default.createElement(
                  'div',
                  { className: classnames('rlpSection', 'rlpComponentMarkup', 'rlpCustomComponentMarkup', styles$7.rlpSection, styles$7.rlpComponentMarkup, styles$7.rlpCustomComponentMarkup) },
                  React__default.createElement(
                    'h6',
                    { className: classnames('rlpContainerSubTitle', styles$7.rlpContainerSubTitle) },
                    this.state.text.customComponentMarkupHeader
                  ),
                  React__default.createElement(
                    CustomComponentMarkup,
                    null,
                    values
                  )
                )
              )
            )
          )
        )
      );
    }
  }]);
  return ReactLiveProps;
}(React.Component);

ReactLiveProps.propTypes = {
  of: PropTypes.func.isRequired,
  docgenInfo: PropTypes.object,
  className: PropTypes.string,
  editableProperties: PropTypes.arrayOf(PropTypes.string),
  additionalTitleText: PropTypes.string,
  hideComponentMarkup: PropTypes.bool,
  hideComponentPreview: PropTypes.bool,
  hidePropsTable: PropTypes.bool,
  customComponentMarkup: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  initialComponent: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]),
  availableTypes: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.func])),
  initialCollapsed: PropTypes.bool,
  hideHeader: PropTypes.bool,
  saveButtonComponent: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  addButtonComponent: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  deleteButtonComponent: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  resetButtonComponent: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  text: PropTypes.shape(text)
};

module.exports = ReactLiveProps;
//# sourceMappingURL=index.js.map
